import React, { Component } from "react";
import Layout from "../components/Layout";
import { StateScreen } from '../components/Screens';
import { Link } from "react-router-dom";
import EvaluateHelmet from "../components/EvaluateHelmet";
import {
  Container,
  Paper,
  Typography,
  Grid,
  Card,
  CardContent,
  Dialog,
  DialogContent,
  DialogActions,
  DialogTitle,
  DialogContentText,
  TextField
} from "@material-ui/core";
import { PrimaryButton, SecondaryButton } from "../components/Buttons";
import EmptyCard from "../components/EmptyCard"
import {get} from "../Utils";

function Event(props) {
  const scoreReportBtn = (props.joined !== true) ?
  <Link to={`/event/${props.event.id}/scorereport`}>
  <PrimaryButton>Score Report</PrimaryButton></Link> : null
  return (
    <Grid item md={12}>
      <Card>
        <CardContent>
          <Grid container spacing={0}>
            <Grid item lg={8}>
              <Typography variant="h5">{props.event.name}</Typography>
              <Typography>{props.event.description}</Typography>
              <br />
              <Link to={`/event/${props.event.id}`}>
              <PrimaryButton>Event Details</PrimaryButton>
              </Link>
              {scoreReportBtn}
            </Grid>
            <Grid item lg={4}>
            <Typography style={{color: "gray"}}>Date</Typography>
            <Typography variant="h6">
              {props.event.date}
            </Typography>
            <Typography style={{color: "gray"}}>Time</Typography>
            <Typography variant="h6">
              {props.event.time}
            </Typography>
            </Grid>
          </Grid>
        </CardContent>
      </Card>
    </Grid>
  );
}

function Buttons(props) {
  const [open, setOpen] = React.useState(false);

  const handleClickOpen = () => {
    setOpen(true);
  };

  const handleClose = () => {
    setOpen(false);
  };
  return (
    <div>
      <Link to="/createEvent">
        <PrimaryButton>Create Event</PrimaryButton>
      </Link>
      <SecondaryButton onClick={handleClickOpen}>Join Event</SecondaryButton>
      <Dialog
        open={open}
        onClose={handleClose}
        aria-labelledby="alert-dialog-title"
        aria-describedby="alert-dialog-description"
      >
        <DialogTitle id="alert-dialog-title">{"Enter Event Code"}</DialogTitle>
        <DialogContent>
          <DialogContentText id="alert-dialog-description">
            <TextField id="invite-code-field" onChange={props.onChange}></TextField>
          </DialogContentText>
        </DialogContent>
        <DialogActions>
          <PrimaryButton onClick={() => {props.join(); handleClose()}}>Join</PrimaryButton>
          <SecondaryButton onClick={handleClose}>Close</SecondaryButton>
        </DialogActions>
      </Dialog>
    </div>
  );
}

class DashBoard extends Component {
  constructor(props) {
    super(props);
    this.state = {
      accountInfo: {},
      joinedEvents: [],
      createdEvents: [],
      inviteCode: "",
      loading: 0
    };

    this.handleJoin = this.handleJoin.bind(this);
  }

  async componentDidMount() {
    let {status, body} = await get("/dashboard");

    if (status === 200) {//okay
      this.setState({
        accountInfo: body.accountInfo,
        joinedEvents: body.joinedEvents,
        createdEvents: body.createdEvents,
        loading: 1
      });
    } else if (status === 404) {//not found
      this.props.history.push("/login")
    } else {
      this.setState({loading: status});
    }
  }

  onChange(event) {
    let newValue = event.target.value;
    this.setState({inviteCode: newValue});
  }

  async handleJoin() {
    let ic = this.state.inviteCode;
    let response = await fetch("/dashboard/join/" + ic, {
      method: "PUT",
      headers: {
          Accept: "application/json",
          "Content-Type": "application/json"
      }
    });

    let status = response.status;
    if (status === 201) {//created
      let body = await response.json();
      this.props.history.push("/event/" + body.id);
    } else if (status === 404) {//not found
      alert("Could not join an event with " + ic + ".");
    } else if (status === 409) {//conflict
      alert("You are already a member of this event.");
    } else if (status === 500) {//internal server error
      alert("Could not reach server. Please try again later.");
    }
  }

  render() {
    if ( this.state.loading !== 1 ) {
        return (
            <StateScreen state={this.state.loading}></StateScreen>
        );
    }

    const { accountInfo, joinedEvents, createdEvents } = this.state;
    var joinedEventList = []
    var createdEventList = [];
    if(joinedEvents.length === 0){
      joinedEventList = (
        <EmptyCard text="If you have an event code, click on the Join Event button above to join."></EmptyCard>
      )
    }else{
      joinedEventList = joinedEvents.map(event => {
        return <Event event={event} key={event.id} joined={true}/>
      });
    }
    if(createdEvents.length === 0){
      createdEventList = (
        <EmptyCard text="Let's create your first event."></EmptyCard>
      )
    }else{
      createdEventList = createdEvents.map(event => {
        return <Event event={event} key={event.id} />;
    });
    }
    
    return (
      <Layout>
        <EvaluateHelmet title="Dashboard"></EvaluateHelmet>
        <Container className="main">
          <Paper
            elevation={3}
            square
            id="dashboardHead"
            className="dashboardCard"
          >
            <Typography variant="h4">
              {accountInfo.firstName} {accountInfo.lastName}
            </Typography>
            <Typography>{accountInfo.email}</Typography>
            <br />
            <Buttons onChange={(e) => this.onChange(e)} join={this.handleJoin}/>
          </Paper>
          <Grid container spacing={3}>
            <Grid item lg={6} sm={6}>
              <Paper elevation={3} className="dashboardCard">
                <Typography variant="h5">My Events</Typography>
                <Grid container spacing={2}>
                {createdEventList}
                </Grid>
              </Paper>
            </Grid>
            <Grid item lg={6} sm={6}>
              <Paper elevation={3} className="dashboardCard">
                <Typography variant="h5">Joined Events</Typography>
                <Grid container spacing={2}>
                  {joinedEventList}
                </Grid>
              </Paper>
            </Grid>
          </Grid>
        </Container>
      </Layout>
    );
  }
}

export default DashBoard;
