import React, { Component } from 'react';
import EvaluateHelmet from "../../components/EvaluateHelmet";
import Layout from '../../components/Layout';
import { StateScreen } from '../../components/Screens';
import { SaveButton, DeleteButton, EditButton, PrimaryButton, SecondaryButton } from "../../components/Buttons";
import DialogBox from "../../components/DialogBox"
import { Container, Card } from '@material-ui/core';
import { Table, TableBody, TableCell, TableContainer, TableHead, TableRow } from '@material-ui/core';
import { Select, MenuItem, Slider, Paper } from '@material-ui/core';
import {get} from "../../Utils";

function EntryRow(props) {
    const entry = props.entry;
    const menuItems = props.menuItems;
    return (
        <TableRow key={entry.title}>
            <TableCell>
                <h4>
                    {entry.title}
                    <EditButton link={`/event/${props.eventId}/entry/${entry.id}`}></EditButton>
                </h4>
                <h5>Team Lead: {entry.teamLeadName}</h5>
            </TableCell>
            <TableCell align="right">
                <Select onChange={props.onChange} value={entry.groupNumber}>
                    {menuItems.map(item => {
                        return <MenuItem key={item.value} value={item.value}>{item.groupName}</MenuItem>;
                    })}
                </Select>
                <DeleteButton onClick={props.delete}></DeleteButton>
            </TableCell>
        </TableRow>
    );
}

function GroupTotalRow(props) {
    return (
        <TableRow key={props.name}>
            <TableCell component="th" scope="row">{props.name}</TableCell>
            <TableCell align="right">{props.groupTotal.judgeCount}</TableCell>
            <TableCell align="right">{props.groupTotal.entryCount}</TableCell>
        </TableRow>
    );
}

function JudgeRow(props) {
    const judge = props.judge;
    const menuItems = props.menuItems;
    let deleteButton = null;
    if (props.hasOwnProperty("delete")) {
        deleteButton = (<DeleteButton onClick={props.delete}></DeleteButton>);
    }

    return (
        <TableRow>
            <TableCell>
                <h4>{judge.name}</h4>
            </TableCell>
            <TableCell align="right">
                <Select onChange={props.onChange} value={judge.groupNumber}>
                    {menuItems.map(item => {
                        return <MenuItem key={item.value} value={item.value}>{item.groupName}</MenuItem>;
                    })}
                </Select>
                {deleteButton}
            </TableCell>
        </TableRow>
    );
}

class AssignGroups extends Component {

    emptyGroupTotal = {
        entryCount: 0,
        judgeCount: 0
    };

    constructor(props) {
        super(props)

        this.state = {
            eventId: this.props.match.params.eventId,
            loading: 0,
            entries: [],
            judges: [],
            groupTotals: [],
            totalGroups: 1,
            popup: {
                isOpen: false,
                name: null,
                type: null,
                id: null
            }
        }
        this.handleChangeTotalGroups = this.handleChangeTotalGroups.bind(this);
    }

    async componentDidMount() {
        let {status, body} = await get(`/event/${this.state.eventId}/assigngroups`);
        if (status === 200) {
            var entries = body.entries;
            var judges = body.judges;
            var judgeNames = body.judgeNames;

            //link names to judges:
            for (let i = 0; i < judges.length; i++) {
                judges[i].name = judgeNames[i];
            }

            //init group totals:
            let [groupTotals, totalGroups] = this.countGroupTotals(judges, entries);

            this.setState({
                entries: entries, 
                judges: judges, 
                totalGroups: totalGroups, 
                groupTotals: groupTotals, 
                loading: 1
            });
        } else {
            this.setState({loading: status});
        }
    }

    /**
     * Copies an object.
     */
    copy(object) {
      return JSON.parse(JSON.stringify(object));
    }

    countGroupTotals(judges, entries) {
        //Initialize group totals:
        let groupTotals = [];
        for (let i = 0; i < 5; i++) {
            groupTotals.push(this.copy(this.emptyGroupTotal));
        }

        //calculate for judges:
        let totalGroups = 1;
        for (let i = 0; i < judges.length; i++) {
            let gn = judges[i].groupNumber;
            if (gn !== -1) {
                groupTotals[gn].judgeCount += 1;
                if (gn >= totalGroups) {
                    totalGroups = gn + 1;
                }
            }
        }

        //calculate for entries:
        for (let i = 0; i < entries.length; i++) {
            let gn = entries[i].groupNumber;
            groupTotals[gn].entryCount += 1;
            if (gn >= totalGroups) {
                totalGroups = gn + 1;
            }
        }

        return [groupTotals, totalGroups];
    }

    groupNumberToLetter(num) {
        let letter = num + 65;
        if (letter === 64) {
            letter = "none";
        } else {
            letter = String.fromCharCode(letter);
        }
        return letter;
    }

    //Change the entry group number
    async handleChangeEntry(event, entryId) {
        let entries = this.state.entries;
        let groupTotals = this.state.groupTotals;
        let newGroupNumber = event.target.value;

        for (let i = 0; i < entries.length; i++) {
            if (entries[i].id === entryId) {
                //decrement old group number:
                groupTotals[entries[i].groupNumber].entryCount -= 1;

                //change group number:
                entries[i].groupNumber = newGroupNumber;

                //increment new group number:
                groupTotals[entries[i].groupNumber].entryCount += 1;
            }
        }
        this.setState({ entries: entries, groupTotals: groupTotals });
    }

    // Change the Group number on top
    async handleChangeTotalGroups(event, newValue) {
        let judges = this.state.judges;
        let entries = this.state.entries;

        //limit judges:
        for (let i = 0; i < judges.length; i++) {
            if ( judges[i].groupNumber >= newValue ) {
                judges[i].groupNumber = 0;
            }
        }

        //limit entries:
        for (let i = 0; i < entries.length; i++) {
            if ( entries[i].groupNumber >= newValue ) {
                entries[i].groupNumber = 0;
            }
        }

        //recalulate group totals:
        let groupTotals = this.countGroupTotals(judges, entries)[0];
        this.setState({
            judges: judges,
            entries: entries, 
            groupTotals: groupTotals,
            totalGroups: newValue
        });
    }

    // Change the judge group number
    async handleChangeJudge(event, judgeId) {
        let judges = this.state.judges;
        let groupTotals = this.state.groupTotals;
        let newGroupNumber = event.target.value;

        for (let i = 0; i < judges.length; i++) {
            if (judges[i].id === judgeId) {
                //decrement old group number:
                if (judges[i].groupNumber !== -1) {
                    groupTotals[judges[i].groupNumber].judgeCount -= 1;
                }

                //change group number:
                judges[i].groupNumber = newGroupNumber;

                //increment new group number:
                if (judges[i].groupNumber !== -1) {
                    groupTotals[judges[i].groupNumber].judgeCount += 1;
                }
            }
        }
        this.setState({ judges: judges, groupTotals: groupTotals });
    }

    //Remove the Entry in the databse as well as the front end 
    async handleDeleteEntry(entryId) {
        let popup = { isOpen: false, type: "", id: null }
        this.setState({popup: popup})

        const response = await fetch("/event/" + this.state.eventId + "/assigngroups/deleteentry/" + entryId,
            {
                method: "DELETE",
                headers: {
                    "Accept": "application/json",
                    "Content-Type": "application/json"
                }
            }
        );

        let status = response.status;
        if (status === 200) {//okay
            let entries = this.state.entries;
            let groupTotals = this.state.groupTotals;

            for (let i = 0; i < entries.length; i++) {
                if (entries[i].id === entryId) {
                    let deletedEntry = entries.splice(i, 1)[0];
                    groupTotals[deletedEntry.groupNumber].entryCount -= 1;
                }
            }
            this.setState({entries: entries, groupTotals: groupTotals})
        }
        else if (status === 404){//not found
            alert("Entry not found")
        }
        else if (status === 500){//internal server error
            alert("Internal Server Error")
        }
    }

    // Remove the Judge in the database as well as the front end 
    async handleDeleteJudge(judgeId) {
        let popup = { isOpen: false, type: "", id: null }
        this.setState({popup: popup})

        const response = await fetch("/event/" + this.state.eventId + "/assigngroups/deletejudge/" + judgeId,
            {
                method: "DELETE",
                headers: {
                    "Accept": "application/json",
                    "Content-Type": "application/json"
                }
            }
        );

        let status = response.status;
        if (status === 200) {//okay
            let judges = this.state.judges;
            let groupTotals = this.state.groupTotals;

            for (let i = 0; i < judges.length; i++) {
                if (judges[i].id === judgeId) {
                    let deletedJudge = judges.splice(i, 1)[0];
                    groupTotals[deletedJudge.groupNumber].judgeCount -= 1;
                }
            }
            this.setState({judges: judges, groupTotals: groupTotals})
        }
        else if (status === 400) {//bad request
            alert("Chair cannot be deleted");
        }
        else if (status === 404){//not found
            alert("Judge not found")
        }
        else if (status === 500){//internal server error
            alert("Internal Server Error")
        }
    }

    async handleSubmit() {
        const eventUrl = "/event/" + this.state.eventId;
        const entries = this.state.entries;
        let judges = this.state.judges;

        //Add role type ot judges:
        judges[0].role_type = 'C';
        for (let i = 1; i < judges.length; i++) {
            judges[i].role_type = 'J';
        }

        //Create save object map:
        var saveObject = {};
        saveObject.entries = entries;
        saveObject.judges = judges;

        const response= await fetch(eventUrl + "/assigngroups",
            {
                method: "POST",
                headers: {
                    "Accept": "application/json",
                    "Content-Type": "application/json"
                },
                body: JSON.stringify(saveObject)
            }
        );
        let status = response.status;
        if (status === 200) {
            this.props.history.push(eventUrl);
        }
        else if (status === 404){
            alert("Event not found")
        }
        else if (status === 500){
            alert("Internal Server Error")
        }
    }

    setUpPopup() {
        //set up action:
        let popup = this.state.popup
        let popupAction = null;
        let popupMsg = "Would you like to immediately and permanently ";
        if (popup.type === "Entry") {
            popupAction = () => this.handleDeleteEntry(popup.id);
            popupMsg += "delete \"" + popup.name + "\"?";
        } else if (popup.type === "Judge") {
            popupAction = () => this.handleDeleteJudge(popup.id);
            popupMsg += "remove " + popup.name + " as a judge and any scores they may have created?";
        }

        //set up open:
        const openPopup = (name, type, id) => {
            let popup = {};
            popup.isOpen = true;
            popup.name = name;
            popup.type = type;
            popup.id = id;
            this.setState({ popup: popup })
        };

        //set up close:
        const close = () => {
            let popup = { isOpen: false, type: "", id: null }
            this.setState({popup: popup})
        };

        //set up buttons:
        let deleteButton = <SecondaryButton onClick={popupAction}>Delete</SecondaryButton>;
        let cancelButton = <PrimaryButton onClick={close}>Cancel</PrimaryButton>;

        //set up dialog box:
        let dialogBox = 
            <DialogBox 
                open={popup.isOpen} 
                handleClose={close} 
                message={popupMsg} 
                leftButton={cancelButton} 
                rightButton={deleteButton} 
                title={"Delete " + popup.type + "?"} 
                handleAction={popupAction} 
            />;

        return [dialogBox, openPopup];
    }

    render() {
        if (this.state.loading !== 1) {
            return (
                <StateScreen state={this.state.loading}></StateScreen>
            );
        }
        var entryRows = [];
        var judgeRows = [];
        var menuItems = [];
        var chairMenuItems = [];
        var groupTotalRows = [];

        const {eventId, entries, judges, groupTotals} = this.state;

        // Push items on select
        chairMenuItems.push({value: -1, groupName: this.groupNumberToLetter(-1)});
        for (let i = 0; i < this.state.totalGroups; i++) {
            let item = { 
                value: i, 
                groupName: this.groupNumberToLetter(i)
            };
            menuItems.push(item);
            chairMenuItems.push(item);
            let name = "Group " + this.groupNumberToLetter(i);
            groupTotalRows.push(<GroupTotalRow key={i} name={name} groupTotal={groupTotals[i]}/>)
        }

        // build dialog box:
        let [dialogBox, openPopup] = this.setUpPopup();

        // build list of entry cards:
        for (let i = 0; i < entries.length; i++) {
            entryRows.push(<EntryRow key={i} entry={entries[i]} eventId={eventId} menuItems={menuItems} onChange={e => this.handleChangeEntry(e, entries[i].id)} delete={() => openPopup(entries[i].title, "Entry", entries[i].id)}/>);
        }

        // include the judge list in the front end
        judgeRows.push(<JudgeRow key={0} judge={judges[0]} menuItems={chairMenuItems} onChange={e => this.handleChangeJudge(e, judges[0].id)}/>);
        for (let i = 1; i < judges.length; i++) {
            judgeRows.push(<JudgeRow key={i} judge={judges[i]} menuItems={menuItems} onChange={e => this.handleChangeJudge(e, judges[i].id)} delete={() => openPopup(judges[i].name, "Judge", judges[i].id)}/>);
        }

        return (
            <Layout>
                <EvaluateHelmet title="Assign Groups"></EvaluateHelmet>
                <Container className="main">
                    <h1>Assign Judges and Entries to Groups</h1>
                    <br />
                    <h3>Number of Groups</h3>
                    <Card>
                        <br /><br />
                        <div style={{ display: 'flex', justifyContent: 'center', alignItems: 'center' }}>
                            <Slider
                                style={{ width: '60%' }}
                                defaultValue={this.state.totalGroups}
                                aria-labelledby="discrete-slider"
                                valueLabelDisplay="on"
                                step={1}
                                marks
                                min={1}
                                max={5}
                                onChangeCommitted={this.handleChangeTotalGroups}
                            />
                        </div>
                    </Card>
                    <br />
                    <div>
                        <h3>Judges:</h3>
                        <TableContainer component={Paper}>
                            <Table>
                                <TableBody>
                                    {judgeRows}
                                </TableBody>
                            </Table>
                        </TableContainer>
                    </div>
                    <br />
                    <div>
                        <h3>Entries:</h3>
                        <TableContainer component={Paper}>
                            <Table>
                                <TableBody>
                                    {entryRows}
                                </TableBody>
                            </Table>
                        </TableContainer>
                    </div>
                    <br />
                    <div>
                        <h3>Summary:</h3>
                        <TableContainer component={Paper}>
                            <Table>
                                <TableHead>
                                    <TableRow>
                                        <TableCell><h5>Group</h5></TableCell>
                                        <TableCell align="right"><h5>Number of Judges</h5></TableCell>
                                        <TableCell align="right"><h5>Number of Entries</h5></TableCell>
                                    </TableRow>
                                </TableHead>
                                <TableBody>
                                    {groupTotalRows}
                                </TableBody>
                            </Table>
                        </TableContainer>
                    </div>
                    <br />
                    <SaveButton onClick={() => this.handleSubmit()}></SaveButton>
                    {dialogBox}
                </Container>
            </Layout>
        );
    }
}

export default AssignGroups;
