import React, { Component } from "react";
import Layout from "../../components/Layout";
import { StateScreen } from '../../components/Screens';
import EvaluateHelmet from "../../components/EvaluateHelmet";
import { Button, TextField, Container } from "@material-ui/core";
import {
  Table,
  TableBody,
  TableCell,
  TableHead,
  TableRow
} from "@material-ui/core";
import {
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions
} from "@material-ui/core";
import {
  PrimaryButton,
  SecondaryButton,
  AddButton,
  SaveButton,
  DeleteButton
} from "../../components/Buttons";
import ArrowRightIcon from '@material-ui/icons/ArrowRight';
import "../../static/styles/style.scss";
import {get} from "../../Utils";

/* Components */

/**
 * A category row in the table.
 *
 * @param {dict} props contains the following keys:
 * @param {dict} catValue category containing id, name, maxScore, and description
 * @param {string} level enumerated value to show relation to other rows
 * @param {function} delete removes given category from the state
 * @param {function} onChange updates state when input value is changed
 * @param {function} addSub (optional) adds a sub category to the given category
 */
function CategoryTableItem(props) {
  //check if add sub button is needed:
  var addSubButton = null;
  if (props.addSub != null) {//if has addSub function
    //then it needs an Add Sub Category Button:
    addSubButton = <AddSubCategoryButton onClick={props.addSub} />;
  }

  //check if max score field should be disabled:
  var disableMaxScore = null;
  if (props.catValue.subCategories.length !== 0) {//if has sub categories
    //disable max score field:
    disableMaxScore = "yes";
  }

  //check if max score has an error:
  var errorMaxScore = null;
  var mshelperText = "";
  if (Number(props.catValue.maxScore) === 0) {
    //if max score is zero
    //mark as error:
    errorMaxScore = true;
    mshelperText = "Invalid value.";
  }

  return (
    <TableRow>
      <TableCell>{props.levelIcon}{props.level}</TableCell>
      <TableCell>
        <TextField
          required
          name="name"
          size="small"
          placeholder="Name"
          value={props.catValue.name}
          onChange={props.onChange}
          inputProps={{ size: 25 }}
        />
      </TableCell>
      <TableCell>
        <TextField
          name="maxScore"
          type="number"
          size="small"
          disabled={disableMaxScore}
          error={errorMaxScore}
          helperText={mshelperText}
          value={props.catValue.maxScore}
          onChange={props.onChange}
          inputProps={{ min: 0, max: 100 }}
        />
      </TableCell>
      <TableCell>
        <TextField
          name="description"
          size="small"
          placeholder="Description"
          value={props.catValue.description}
          onChange={props.onChange}
          inputProps={{ size: 60 }}
        />
      </TableCell>
      <TableCell>
        <DeleteButton aria-label="delete category" onClick={props.delete} />
        {addSubButton}
      </TableCell>
    </TableRow>
  );
}

/**
 * An "Add Sub Category" button.
 *
 * @param {dict} props contains following:
 * @param {function} onClick function to execute when clicked
 */
function AddSubCategoryButton(props) {
  return <AddButton aria-label="add sub category" onClick={props.onClick} />;
}

/* Class */

class CreateRubric extends Component {
  /* empty models */

  emptyRubric = {
    categories: [],
    totalMaxScore: 10
  };

  emptyCategory = {
    name: "",
    description: "",
    maxScore: 10,
    subCategories: [],
    scoreUpToDate: true
  };

  /* conctructor */

  constructor(props) {
    super(props);
    this.state = {
      eventId: props.match.params.eventId,
      eventName: "",
      isEditing: false,
      loading: 0,
      popup: {
        isOpen: false,
        title: "",
        msg: ""
      }
    };

    this.onChange = this.onChange.bind(this);
  }

  /* funcitons */

  /**
   * Automatically called at page load.
   * POSTs a new rubric to build categories under.
   */
  async componentDidMount() {
    //get event name:
    let {status, body} = await get(`/event/${this.state.eventId}/createrubric/eventname`);

    if ( status === 200 ) {//okay
      this.setState({eventName: body.name});
    } else {//not found
      this.setState({loading: status});
      return;
    }

    let path = this.props.location.pathname;
    let createRegex = /\/createrubric/;
    if ( createRegex.test(path) ) {//if creating new rubric
      //create new rubric:
      let rubric = this.copy(this.emptyRubric);
  
      //add new category to rubric:
      let newCat = this.copy(this.emptyCategory);
      rubric.categories.push(newCat);
      this.setState({ rubric: rubric, loading: 1 });
    } 
    else {//if editting old rubric
      let {status, body: rubric} = await get(`/event/${this.state.eventId}/createrubric/`);

      if ( status === 200 ) {//okay
        this.setState({ rubric, loading: 1, isEditing: true });
      } 
      else {
        this.setState({loading: status});
      }
    }
  }

  /**
   * Appends a category to the rubric's list
   * of categories.
   */
  async addCat() {
    //create local category:
    var newCat = this.copy(this.emptyCategory);

    //add to rubric categories:
    var rubric = this.state.rubric;
    rubric.categories.push(newCat);

    //update max score:
    rubric = this.recalculateMaxScore(rubric);

    //update rubric:
    this.setState({ rubric: rubric });
  }

  /**
   * Adds a sub category to a category with a
   * given index.
   *
   * @param {number} parIndex Index of parent category to add sub category to
   */
  async addSub(parIndex) {
    //create local sub category:
    var newSub = this.copy(this.emptyCategory);

    //add to rubric:
    var rubric = this.state.rubric;
    var cats = rubric.categories;
    cats[parIndex].subCategories.push(newSub);

    //recalculate:
    cats[parIndex].scoreUpToDate = false;
    rubric = this.recalculateMaxScore(rubric);

    //update state:
    this.setState({ rubric: rubric });
  }

  /**
   * Builds a table body out of the state categories.
   */
  buildTable() {
    const categories = this.state.rubric.categories;
    var tableItems = [];
    for (var i = 0; i < categories.length; i++) {
      //iterate through categories:
      //add category to table:
      var level = (i + 1).toString(10);
      const c = this.copy(categories[i]);
      const index = i;
      tableItems.push(
        <CategoryTableItem
          level={level}
          catValue={c}
          delete={() => this.removeCat(index, -1)}
          addSub={() => this.addSub(index)}
          onChange={e => this.onChange(e, index, -1)}
        />
      );

      //check if has sub categories and add if so:
      const subs = categories[i].subCategories;
      for (var k = 0; k < subs.length; k++) {
        //iterate through sub categories:
        level = (i + 1) + "." + (k + 1);
        const s = this.copy(subs[k]);
        const sindex = k;
        tableItems.push(
          <CategoryTableItem
            level={level}
            levelIcon={<ArrowRightIcon fontSize="small"/>}
            catValue={s}
            delete={() => this.removeCat(index, sindex)}
            onChange={e => this.onChange(e, index, sindex)}
          />
        );
      }
    }
    return tableItems;
  }

  /**
   * Copies an object.
   */
  copy(object) {
    return JSON.parse(JSON.stringify(object));
  }

  /**
   * Updates the state rubric based on the change.
   *
   * @param {SyntheticEvent} e event that occured
   * @param {number} catIndex index of category that the event happened to
   * @param {number} subIndex index of sub category that the event happened to
   */
  async onChange(e, catIndex, subIndex) {
    e.persist();

    //find category with indices:
    var rubric = this.state.rubric;
    var cats = rubric.categories;

    var changedCat;
    if (subIndex === -1) {
      changedCat = cats[catIndex];
    } else {
      changedCat = cats[catIndex].subCategories[subIndex];
    }

    //change its value:
    switch (e.target.name) {
      case "name":
        changedCat.name = e.target.value;
        break;
      case "maxScore":
        changedCat.maxScore = e.target.value;
        if (subIndex !== -1) {
          cats[catIndex].scoreUpToDate = false;
        }
        rubric = this.recalculateMaxScore(rubric);
        break;
      case "description":
        changedCat.description = e.target.value;
        break;
      default:
        break;
    }

    //update state:
    this.setState({ rubric: rubric });
  }

  /**
   * Recalculates the max score of the rubric and all categories in the
   * rubric, if they have been marked as not scoreUpToDate.
   *
   * @param {dict} rubric to be updated
   * @returns {dict} rubric with updated max score values
   */
  recalculateMaxScore(rubric) {
    var cats = rubric.categories;

    rubric.totalMaxScore = 0;
    for (var i = 0; i < cats.length; i++) {
      //for each category
      if (!cats[i].scoreUpToDate) {
        //if cat not up to date
        //recalculate max score:
        cats[i].maxScore = 0;
        var subCats = cats[i].subCategories;
        for (var k = 0; k < subCats.length; k++) {
          cats[i].maxScore += parseInt(subCats[k].maxScore, 10);
        }
        cats[i].scoreUpToDate = true;
      }

      //add to rubric total:
      rubric.totalMaxScore += parseInt(cats[i].maxScore, 10);
    }

    //update state:
    return rubric;
  }

  /**
   * Removes all categories from the rubric.
   */
  async removeAllCats() {
    var rubric = this.state.rubric;

    //remove categories:
    rubric.categories = [];

    //add one new category:
    var newCat = this.copy(this.emptyCategory);
    rubric.categories.push(newCat);

    //set rubric total max score to zero:
    rubric.totalMaxScore = 10;

    //update rurbic:
    this.setState({ rubric: rubric });
  }

  /**
   * Removes a category or a sub category. If a category
   * is removed, all of its sub categories are removed too.
   *
   * To remove a category, the given subIndex should be
   * equal to -1.
   *
   * @param {number} catIndex to be removed
   * @param {number} subIndex to be removed
   */
  async removeCat(catIndex, subIndex) {
    var rubric = this.state.rubric;
    var cats = rubric.categories;

    //remove:
    if (subIndex === -1) {
      //if removing category
      //remove category:
      cats.splice(catIndex, 1);
    } else {
      //if removing sub category
      //remove sub category:
      cats[catIndex].subCategories.splice(subIndex, 1);

      //mark parent as not up to date:
      cats[catIndex].scoreUpToDate = false;
    }

    //check if at least one category:
    if (cats.length === 0) {
      var newCat = this.copy(this.emptyCategory);
      rubric.categories.push(newCat);
    }

    //recalculate:
    rubric = this.recalculateMaxScore(rubric);

    //set updated rubric:
    this.setState({ rubric: rubric });
  }

  /**
   * Validates the form, then posts the new rubric
   * and all of its categories.
   */
  async save() {
    var rubric = this.state.rubric;
    var cats = rubric.categories;

    //verify rubric:
    var blankName = false;
    var zeroScore = false;
    for (var i = 0; i < cats.length; i++) {
      //iterate through categories
      if (cats[i].name === "") {
        //if no category name
        blankName = true;
        break;
      } else if (Number(cats[i].maxScore) === 0) {
        //if max score has zero value
        zeroScore = true;
        break;
      }

      var subCats = cats[i].subCategories;
      for (var k = 0; k < subCats.length; k++) {
        //iterate through sub categories
        if (subCats[k].name === "") {
          //if no category name
          blankName = true;
          break;
        } else if (Number(subCats[k].maxScore) === 0) {
          //if max score has zero value
          zeroScore = true;
          break;
        }
      }
      if (blankName || zeroScore) {
        //if break in sub iteration
        break;
      }
    }

    //show popup if failed:
    var popup;
    if (blankName) {
      popup = this.state.popup;
      popup.isOpen = true;
      popup.title = "Invalid Rubric";
      popup.msg = "One or more category names was left blank.";
      this.setState({ popup: popup });
      return false;
    } else if (zeroScore) {
      popup = this.state.popup;
      popup.isOpen = true;
      popup.title = "Invalid Rubric";
      popup.msg = "One or more max scores has a value of zero.";
      this.setState({ popup: popup });
      return false;
    }

    //post/put rubric:
    let eventId = this.state.eventId;
    let method = (this.state.isEditing) ? "PUT" : "POST";
    let response = await fetch("/event/" + eventId + "/createrubric", {
      method: method,
      headers: {
        Accept: "application/json",
        "Content-Type": "application/json"
      },
      body: JSON.stringify(rubric)
    });

    if (response.status === 403) {//forbidden
      let popup = this.state.popup;
      popup.isOpen = true;
      popup.title = "Error 403 Forbidden";
      popup.msg = "This event already has a rubric.";
      this.setState({ popup: popup });
      return false;
    } else if (response.status === 404) {//not found
      let popup = this.state.popup;
      popup.isOpen = true;
      popup.title = "Error 404 Not Found";
      popup.msg = "This event does not exist.";
      this.setState({ popup: popup });
      return false;
    } else if (response.status === 422) {//unprocessable entity
      let popup = this.state.popup;
      popup.isOpen = true;
      popup.title = "Error 422 Unprocessable Entity";
      popup.msg = "The previous version of this rubric has already been used to grade entries. Please remove all grades before changing the rubric.";
      this.setState({ popup: popup });
      return false;
    }

    this.props.history.push("/event/" + eventId);
    return true;
  }

  render() {
    //state screen:
    if ( this.state.loading !== 1 ) {
      return (
          <StateScreen state={this.state.loading}></StateScreen>
      );
    }

    var tableItems = this.buildTable();

    //set title:
    let title = (this.state.isEditing) ? "Edit Rubric" : "Create a Rubric";
    title = (this.state.eventName === "") ? title : this.state.eventName + ": " + title;

    //set up popup:
    const popup = this.state.popup;
    const dialogClose = () => {
      var popup = this.state.popup;
      popup.isOpen = false;
      this.setState({ popup: popup });
    };

    return (
      <Layout>
        {/* Helmet is for the <head> */}
        <EvaluateHelmet title={title}></EvaluateHelmet>
        {/* Main Content Here */}
        <Container className="main">
          <div className="container">
            <h1>{title}</h1>
          </div>
          <div className="container buttonSpacing">
            <PrimaryButton
              aria-label="add category"
              onClick={() => this.addCat()}
            >
              Add Category
            </PrimaryButton>
            <SecondaryButton
              aria-label="delete all categories"
              onClick={() => this.removeAllCats()}
            >
              Delete All
            </SecondaryButton>
            <SaveButton onClick={() => this.save()} />
          </div>
          <div 
            className="container"
            id="rubricCreationForm"
          >
            <Table aria-label="rubric table">
              <TableHead>
                <TableRow>
                  <TableCell>
                    <b>Level</b>
                  </TableCell>
                  <TableCell>
                    <b>Category Name</b>
                  </TableCell>
                  <TableCell>
                    <b style={{ "white-space": "nowrap"}}>Max Score: {this.state.rubric.totalMaxScore}</b>
                  </TableCell>
                  <TableCell>
                    <b>Description (Optional)</b>
                  </TableCell>
                  <TableCell></TableCell>
                </TableRow>
              </TableHead>
              <TableBody>{tableItems}</TableBody>
            </Table>
          </div>
          <div>
            <Dialog
              open={popup.isOpen}
              onClose={dialogClose}
              aria-labelledby="alert-dialog-slide title"
            >
              <DialogTitle id="alert-dialog-slide title">
                {popup.title}
              </DialogTitle>
              <DialogContent>{popup.msg}</DialogContent>
              <DialogActions>
                <Button color="primary" onClick={dialogClose}>
                  Okay
                </Button>
              </DialogActions>
            </Dialog>
          </div>
        </Container>
      </Layout>
    );
  }
}

export default CreateRubric;
