import React, { Component } from "react";
import EvaluateHelmet from "../../components/EvaluateHelmet";
import Layout from "../../components/Layout";
import { StateScreen } from '../../components/Screens';
import { Container, Grid } from "@material-ui/core";
import { PrimaryButton, SaveButton } from "../../components/Buttons";
import TextFields from "../../components/TextFields";
import {get} from "../../Utils";

function EventDetails(props) {
  let event = props.event;

  return (
    <div>
      <Grid item xs={12}><h1>Welcome to {event.name}!</h1></Grid>
      <Grid item xs={12}><h5>Location: {event.location}</h5></Grid>
      <Grid item xs={12}>
        <h5>
          Date:&nbsp;{event.date + " @ " + event.time}
        </h5>
      </Grid>
      <Grid item xs={12}><h5>Description: {event.description}</h5></Grid>
      <Grid item xs={12}>
        After submitting, you will receive a confirmation email within the next 10 to 15 minutes. If you do not receive an email, please contact the organizer of your event
      </Grid>
      <Grid item xs={12}><em>Required fields *</em></Grid>
    </div>
  );
}

class EventRegister extends Component {

  constructor(props) {
    super(props);
    this.state = {
      event: null,
      entryId: null,
      teamName: "",
      title: "",
      description: "",
      teamLeadName: "",
      teamLeadEmail: "",
      teamMemberNames: "",
      loading: 0,
      editMode: false
    };
    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  async componentDidMount() {
    if ('entryId' in this.props.match.params) {// if editing an entry
      // fetch entry data:
      let {eventId, entryId} = this.props.match.params;
      let {status, body: entry} = await get(`/event/${eventId}/entry/${entryId}`);

      // handle response
      if (status === 200) {//okay
        this.setState({
          event: {id: eventId},
          entryId,
          teamName: entry.teamName,
          title: entry.title,
          description: entry.description,
          teamLeadName: entry.teamLeadName,
          teamLeadEmail: entry.teamLeadEmail,
          teamMemberNames: entry.teamMemberNames,
          editMode: true,
          loading: 1
        });
      } else {
        this.setState({loading: status})
      }
    } else {// if creating a new entry
      // fetch event data:
      let {eventId} = this.props.match.params;
      let {status, body: event} = await get(`/event/${eventId}/eventregister`);

      // handle response
      if (status === 200) {//okay
        this.setState({
          event, 
          loading: 1
        });
      } else {
        this.setState({loading: status})
      }
    }
  }

  handleChange(event) {
    this.setState({
      [event.target.name]: event.target.value
    });
  }

  async handleSubmit() {
    this.setState({loading: -1});
    const entry = this.state;
    const eventId = this.state.event.id;
    const entryId = this.state.entryId;
    let uri = `/event/${eventId}/eventregister`;
    let method = "POST";
    if (this.state.editMode) {
      uri = `/event/${eventId}/entry/${entryId}`;
      method = "PUT";
    }

    let response = await fetch(uri, {
      method,
      headers: {
        Accept: "application/json",
        "Content-Type": "application/json"
      },
      body: JSON.stringify(entry)
    });
    let status = response.status;
    if(status === 200 || status === 201){
      if (this.state.editMode) {
        this.props.history.goBack();
      } else {
        this.props.history.push(`/event/${eventId}`);
      }
    } else if (status === 500) {//internal server error
      alert("Could not reach server. Please try again later.");
    } else {
      alert("Error " + status);
    }
    this.setState({loading: 1});
  }

  render() {
    if ( this.state.loading !== 1 ) {
      return (
        <StateScreen state={this.state.loading}></StateScreen>
      );
    }

    const { event } = this.state;
    let eventDetails = null;
    let button = (<SaveButton onClick={() => this.handleSubmit()}></SaveButton>);
    if (!this.state.editMode) {
      eventDetails = (<EventDetails event={event}></EventDetails>);
      button = (<PrimaryButton onClick={() => this.handleSubmit()}>Submit</PrimaryButton>);
    }

    return (
      <Layout>
        <EvaluateHelmet title="Register for an Event"></EvaluateHelmet>
        <Container component="main" className="main" maxWidth="md">
          <Grid container spacing={2}>
            {eventDetails}
            <Grid item xs={12}>
              <Grid container spacing={2} id="eventRegisterForm">
                <Grid item xs={12}>
                  <TextFields
                    required
                    fullWidth
                    id="entry_title"
                    name="title"
                    label="Entry Title"
                    placeholder="e-Valuate: Online Judging and Interactive Rubric Plarform."
                    defaultValue={this.state.title}
                    onChange={this.handleChange}
                  />
                </Grid>
                <Grid item xs={12}>
                  <TextFields
                    fullWidth
                    id="entry_details"
                    name="description"
                    label="Entry Description/Abstract"
                    placeholder="This project aims to ..."
                    multiline
                    rowsMax="6"
                    defaultValue={this.state.description}
                    onChange={this.handleChange}
                  />
                </Grid>
                <Grid item xs={12}>
                  <TextFields
                    fullWidth
                    id="team_name"
                    name="teamName"
                    label="Team Name"
                    placeholder="This can be your school's name."
                    defaultValue={this.state.teamName}
                    onChange={this.handleChange}
                  />
                </Grid>
                <Grid item xs={12}>
                  <TextFields
                    required
                    fullWidth
                    id="lead_name"
                    name="teamLeadName"
                    label="Team Lead's Full Name"
                    placeholder="John Doe"
                    defaultValue={this.state.teamLeadName}
                    onChange={this.handleChange}
                  />
                </Grid>
                <Grid item xs={12}>
                  <TextFields
                    required
                    fullWidth
                    id="lead_email"
                    name="teamLeadEmail"
                    label="Team Lead's Email"
                    placeholder="example@email.com"
                    type="email"
                    defaultValue={this.state.teamLeadEmail}
                    onChange={this.handleChange}
                  />
                </Grid>
                <Grid item xs={12}>
                  <TextFields
                    fullWidth
                    id="team_members"
                    name="teamMemberNames"
                    label="Team Members"
                    placeholder="Member 1, Member 2, Member 3, etc."
                    defaultValue={this.state.teamMemberNames}
                    onChange={this.handleChange}
                  />
                </Grid>
                <Grid item xs={12}>
                  {button}
                </Grid>
              </Grid>
            </Grid>
          </Grid>
        </Container>
      </Layout>
    );
  }
}

export default EventRegister;
