package com.evaluate.models;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;

@Entity
@Table(name = "Account")
public class Account implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	@Column(name = "id", nullable = false, unique = true)
	private long id;

	@NotNull
	@Column(name = "email", nullable = false, unique = true)
	private String email;

	@NotNull
	@Column(name = "encrypted_password", nullable = false)
	@JsonProperty(access = Access.WRITE_ONLY)
	private String encryptedPassword;

	@Column(name = "first_name")
	private String firstName;

	@Column(name = "last_name")
	private String lastName;

	@OneToMany(mappedBy = "account")
	private List<Role> roles;

	@Column(name = "reset_password_code")
	@JsonProperty(access = Access.WRITE_ONLY)
	private String encryptedResetCode;

	public Account() {
		super();
		email = null;
		encryptedPassword = null;
		firstName = null;
		lastName = null;
		roles = new ArrayList<Role>();
		encryptedResetCode = "";
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getEncryptedPassword() {
		return encryptedPassword;
	}

	public void setEncryptedPassword(String epw) {
		this.encryptedPassword = epw;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}
	
	public String getFullName() {
		return firstName + " " + lastName;
	}
	
	public List<Role> getRoles() {
		return roles;
	}

	@SuppressWarnings("unchecked")
	public void setRoles(List<Role> roles) {
		// swap lists:
		List<Role> oldRoles = this.roles;
		this.roles = new ArrayList<Role>();
		if (roles != null) {
			roles = (List<Role>) ((ArrayList<Role>) roles).clone();
		} else {
			roles = new ArrayList<Role>();
		}

		// remove old references to this:
		for (Role or : oldRoles) {
			if (this.equals(or.getAccount())) {
				or.setAccount(null);
			}
		}

		// set new list:
		this.roles = roles;

		// add new references to this:
		for (Role r : this.roles) {
			if (r.getAccount() == null || !r.getAccount().equals(this)) {
				r.setAccount(this);
			}
		}
	}

	/***
	 * Adds a role to the list of roles and adds the event reference to this
	 * instance if it does not already exist. Returns true if the role is added
	 * successfully and is not a duplicate of another role already in the list.
	 * 
	 * @param role Role to be added
	 * @return True if newly added to the list
	 */
	public boolean addRole(Role role) {
		// error check:
		if (roles.contains(role) || role == null) {
			// do not add
			return false;
		}

		// add to list and save output to return at end:
		boolean output = roles.add(role);

		// reciprocate relation:
		if (!this.equals(role.getAccount())) {
			role.setAccount(this);
		}

		return output;
	}

	public boolean removeRole(Role role) {
		if (role == null) {
			return false;
		}

		// remove relation:
		boolean output = roles.remove(role);

		// remove reciprocal relation:
		if (this.equals(role.getAccount())) {
			role.setAccount(null);
		}

		return output;
	}

	public Role removeRole(int index) {
		// remove relation:
		Role output = roles.remove(index);

		// remove reciprocal relation:
		if (this.equals(output.getAccount())) {
			output.setAccount(null);
		}

		return output;
	}

	public String getEncryptedResetCode() {
		return encryptedResetCode;
	}

	public void setEncryptedResetCode(String encryptedResetCode) {
		this.encryptedResetCode = encryptedResetCode;
	}

	@Override
	public String toString() {
		String output = "Account [";
		output += "\n\tid=" + id;
		output += ",\n\temail=";
		output += (email != null) ? email : "null";
		output += ",\n\tencrypted_password=";
		output += (encryptedPassword != null) ? encryptedPassword : "null";
		output += ",\n\tfirst_name=";
		output += (firstName != null) ? firstName : "null";
		output += ",\n\tlast_name=";
		output += (lastName != null) ? lastName : "null";
		output += ",\n\tNumber of Roles=" + roles.size();
		output += "\n]";
		return output;
	}

}
