package com.evaluate.models;

import javax.persistence.Column;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;
import com.sun.istack.NotNull;

import java.io.*; 
import java.util.*; 

@Entity
@Table(name="Category")
public class Category implements Serializable{
	
	private static final long serialVersionUID = 1L;
	
	@Id
	@NotNull
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	@Column(name="id", nullable=false, unique=true)
	private long id;
	
	@NotNull
	@Column(name="name", nullable=false)
	private String name;
	
	@Column(name="description")
	private String description;
	
	@NotNull
	@Column(name="max_score", nullable=false)
	private int maxScore;
	
	@ManyToOne
	@JoinColumn(name="parent_cat_id")
	@JsonProperty(access = Access.WRITE_ONLY)
	private Category parentCategory;
	
	@OneToMany(mappedBy="parentCategory")
	private List<Category> subCategories;
	
	@OneToMany(mappedBy="template")
	@JsonProperty(access = Access.WRITE_ONLY)
	private Set<Score> scores;
	
	@ManyToOne
	@JoinColumn(name="rubric_id")
	@JsonProperty(access = Access.WRITE_ONLY)
	private Rubric rubric;
	
	@Transient
	private boolean isUpToDate;
	
	//constructors
	public Category() {
		super();
		setName("");
		setDescription(null);
		subCategories = new ArrayList<Category>();
		setMaxScore(0);
		parentCategory = null;
		scores = new HashSet<Score>();
		isUpToDate = true;
	}
	
	public Category(String name) {
		super();
		setName(name);
		setDescription(null);
		subCategories = new ArrayList<Category>();
		setMaxScore(0);
		parentCategory = null;
		scores = new HashSet<Score>();
		isUpToDate = true;
	}
	
	public Category(String name, int maxScore){
		super();
		setName(name);
		setDescription(null);
		subCategories = new ArrayList<Category>();
		setMaxScore(maxScore);
		parentCategory = null;
		scores = new HashSet<Score>();
		isUpToDate = true;
	}
	
	public Category(String name, int maxScore, Category parentCategory){
		super();
		setName(name);
		setDescription(null);
		subCategories = new ArrayList<Category>();
		setMaxScore(maxScore);
		setParentCategory(parentCategory);
		scores = new HashSet<Score>();
		isUpToDate = true;
	}

	//setters and getters
	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		if ( name == null ) {
			this.name = "";
		}else {
			this.name = name;
		}
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		if ( description == null || description.isEmpty()) {
			this.description = null;
		} else {
			this.description = description;
		}
	}

	public int getMaxScore() {
		return maxScore;
	}

	/**
	 * MaxScore can only be set if it's a leaf category.
	 * If it's not, return false.
	 * If the maxScore value is unchanged, return false also.
	 * 
	 * @param maxScore
	 * @return true if successfully changed
	 */
	public boolean setMaxScore(int maxScore) {
		//if this is not a leaf
		if(!subCategories.isEmpty()) {
			//do not change score
			return false;
		} else if(maxScore == this.maxScore) {	//if the value does not change
			return false;
		} else if( maxScore < 0 ) {//if value is less than zero
			return false;
		}
		
		this.maxScore = maxScore;
		recalculateAll();
		return true;
	}

	public Category getParentCategory() {
		return parentCategory;
	}

	public void setParentCategory(Category parentCategory) {
		if( this.parentCategory != null && this.parentCategory.equals(parentCategory) ) {//if parent already set
			//do nothing:
			return;
		}

		Category oldParent = this.parentCategory;
		this.parentCategory = parentCategory;
		
		//remove old references:
		if( oldParent != null && oldParent.getSubCategories().contains(this) ) {
			oldParent.removeSubCategory(this);
		}
		
		//add new references:
		if(parentCategory != null && !parentCategory.getSubCategories().contains(this)) {
			parentCategory.addSubCategory(this);
		}
	}

	public List<Category> getSubCategories() {
		return subCategories;
	}

	@SuppressWarnings("unchecked")
	public void setSubCategories(List<Category> subCategories) {
		List<Category> oldSubCategories = this.subCategories;
		this.subCategories = new ArrayList<Category>();
		
 		if ( subCategories != null ) {//if not null
 			//copy list:
 			subCategories = (List<Category>)((ArrayList<Category>)subCategories).clone();
 		} else {//if null
 			//create empty list:
 			subCategories = new ArrayList<Category>();
 		}

 		//removing old references to this
 		for (Category oldSC : oldSubCategories) {
 			if ( this.equals(oldSC.getParentCategory()) ) {
 				oldSC.setParentCategory(null);
 			}
 		}
 		
 		//set new sub categories:
 		this.subCategories = subCategories;

 		//adding new references to this
 		for (Category newSC : this.subCategories) {
 			if ( newSC.getParentCategory() == null || !newSC.getParentCategory().equals(this) ) {
 				newSC.setParentCategory(this);
 			}
 		}
 		
 		recalculateAll();
	}
	
	public boolean addSubCategory(Category cat) {
		if(this.getSubCategories().contains(cat)) {	//if duplicate
			//do not add
			return false;
		}
		
		boolean output = subCategories.add(cat);

 		//reciprocate relation:
 		if ( cat.getParentCategory() == null || !cat.getParentCategory().equals(this) ) {//if reference isn't already reciprocated
 			cat.setParentCategory(this);
 		}

 		//recalculate points awarded:
 		recalculateAll();		
 		return output;
	}
	
	public Category removeSubCategory(int index) {
		if ( index < 0 || index >= subCategories.size() ) {//if index out of bounds
			return null;
		}
		
		Category categoryRemoved = subCategories.remove(index);

 		//remove reciprocal relation:
 		if ( categoryRemoved.getParentCategory() != null && categoryRemoved.getParentCategory().equals(this) ) {//if reciprocal relation exists
 			categoryRemoved.setParentCategory(null);
 		}

 		//return removed:
 		recalculateAll();
 		return categoryRemoved;
 	}
	
	public boolean removeSubCategory(Category c) {
		//remove relation:
 		if ( !subCategories.remove(c) ) {//if not removed
 			//do nothing:
 			return false;
 		}
 		//if removed:
 		
 		//remove reciprocal relation:
 		if ( c.getParentCategory() != null && c.getParentCategory().equals(this) ) {//if reciprocal relation exists
 			c.setParentCategory(null);
 		}

 		//recalculate all because of change:
 		recalculateAll();
 		return true;
 	}

	public Set<Score> getScores() {
		return scores;
	}

	@SuppressWarnings("unchecked")
	public void setScores(Set<Score> scores) {
		Set<Score> oldScores = this.scores;
		this.scores = new HashSet<Score>();
		
 		if ( scores != null ) {//if not null
 			//copy list:
 			scores = (Set<Score>)((HashSet<Score>)scores).clone();
 		} else {//if null
 			//create empty Set:
 			scores = new HashSet<Score>();
 		}

 		//removing old references to this
 		for (Score oldSC : oldScores) {
 			if ( this.equals(oldSC.getTemplate()) ) {
 				oldSC.setTemplate(null);
 			}
 		}
 		
 		this.scores = scores;

 		//adding new references to this
 		for (Score newSC : this.scores) {
 			if ( newSC.getTemplate() == null || !newSC.getTemplate().equals(this) ) {
 				newSC.setTemplate(this);
 			}
 		}
 		
	}

	public boolean addScore(Score sc) {
		if(this.getScores().contains(sc)) {	//if duplicate
			//do not add
			return false;
		}
		
		if(sc == null) {//if sc is null
			//do nothing
			return false;
		}
		
		boolean output = scores.add(sc);

 		//reciprocate relation:
 		if ( sc.getTemplate() == null || !sc.getTemplate().equals(this) ) {//if reference isn't already reciprocated
 			sc.setTemplate(this);
 		}

 		//recalculate points awarded:	
 		return output;
	}
	
	public boolean removeScore(Score s) {
		//error check:
		if ( s == null ) {
			return false;
		}
		
 		//remove relation:
 		boolean wasRemoved = scores.remove(s);

 		//remove reciprocal relation:
 		if ( s.getTemplate() != null && s.getTemplate().equals(this) ) {//if reciprocal relation exists
 			s.setTemplate(null);
 		}

 		//return if was removed
 		return wasRemoved;
 	}
	
	public Rubric getRubric() {
		return rubric;
	}

	public void setRubric(Rubric rubric) {
		//error check:
		if ( rubric != null && rubric.equals(this.rubric) ) {//if duplicate
			//do nothing:
			return;
		}
		
		//swap:
		Rubric oldRubric = this.rubric;
		this.rubric = rubric;
		
		//remove old reference:
		if ( oldRubric != null && oldRubric.getCategories().contains(this) ) {//if reference to this exists
			oldRubric.removeCategory(this);
		}
		
		//add new reference:
		if ( this.rubric != null && !this.rubric.getCategories().contains(this) ) {//if reference to this does not exist
			this.rubric.addCategory(this);
		}
		
		this.recalculateAll();
	}
	
	@Override
	public String toString() {
		String output = "Category [";
		output += "\n\tid=" + id;
		output += ",\n\tname=";
		output += (name != null) ? name : "null";
		output += ",\n\tdescription=";
		output += (description != null) ? description : "null";
		output += ",\n\tmax_score=" + maxScore;
		output += ",\n\tparent_cat_id=";
		output += (parentCategory != null) ? parentCategory.getId() : "null";
		output += ",\n\tNumber of Sub Categories=" + subCategories.size(); 
		output += ",\n\tNumber of Scores=" + scores.size();
		output += "\n]";
		return output;
	}
 	
 	public void recalculateMaxScore() {
 		if ( isUpToDate ) {//if already up to date
 			//do nothing:
 			return;
 		}
 		if ( subCategories.size() == 0 ) {//if no sub scores
 			//do nothing, because pointsAwarded is not generated:
 			return;
 		}

 		int ms = 0;
 		for (Category sc : subCategories) {
 			sc.recalculateMaxScore();
 			ms += sc.maxScore;
 		}
 		this.maxScore = ms;
 		isUpToDate = true;
 	}
	
	/***
 	 * Recalculates all related points awarded which 
 	 * have changed, including sub scores, parent scores, 
 	 * and scored rubrics. This is done recursively. 
 	 */
 	private void recalculateAll() {
 		isUpToDate = false;
 		if ( parentCategory != null ) {//if has parent
 			//go up a level:
 			parentCategory.recalculateAll();
 		} else if ( rubric != null ) {//if has scored rubric
 			//recalculate from top:
 			rubric.recalculateTotalMaxScore();
 		} else {//if has neither
 			//recalculate from top:
 			recalculateMaxScore();
 		}
 	}

}
