package com.evaluate.models;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;

@Entity
@Table(name = "Entry")
public class Entry implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	@Column(name = "id", nullable = false, unique = true)
	private long id;

	@NotNull
	@Column(name = "title", nullable = false)
	private String title;

	@Column(name = "description", columnDefinition = "TEXT")
	private String description;

	@ManyToOne
	@NotNull
	@JoinColumn(name = "event_id", nullable = false)
	private Event event;

	@OneToMany(mappedBy = "entry")
	@JsonProperty(access = Access.WRITE_ONLY)
	private Set<ScoredRubric> scoredRubrics;

	@Column(name = "team_name")
	private String teamName;

	@NotNull
	@Column(name = "team_lead_name", nullable = false)
	private String teamLeadName;

	@NotNull
	@Column(name = "team_lead_email", nullable = false)
	private String teamLeadEmail;

	@Column(name = "team_member_names")
	private String teamMemberNames;

	@Column(name = "group_number")
	private int groupNumber;

	public Entry() {
		super();
		setTitle("");
		this.description = null;
		this.event = null;
		this.scoredRubrics = new HashSet<ScoredRubric>();
		this.teamName = null;
		this.teamLeadName = null;
		this.teamLeadEmail = null;
		this.teamMemberNames = null;
		this.groupNumber = 0;
	}

	public Entry(String title) {
		super();
		setTitle(title);
		this.description = null;
		this.event = null;
		this.scoredRubrics = new HashSet<ScoredRubric>();
		this.teamName = null;
		this.teamLeadName = null;
		this.teamLeadEmail = null;
		this.teamMemberNames = null;
		this.groupNumber = 0;
	}

	public Entry(String title, String description) {
		super();
		setTitle(title);
		this.description = description;
		this.event = null;
		this.scoredRubrics = new HashSet<ScoredRubric>();
		this.teamName = null;
		this.teamLeadName = null;
		this.teamLeadEmail = null;
		this.teamMemberNames = null;
		this.groupNumber = 0;
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		// error check:
		if (title == null) {// if new title is null
			// set title as empty string instead
			this.title = "";
			return;
		}

		// set new title
		this.title = title;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Event getEvent() {
		return event;
	}

	public void setEvent(Event event) {
		// error check:
		if (event != null && event.equals(this.event)) {// if duplicate
			// do nothing:
			return;
		}

		// swap:
		Event oldEvent = this.event;
		this.event = event;

		// remove old reference:
		if (oldEvent != null && oldEvent.getEntries().contains(this)) {// if reference to this exists
			oldEvent.removeEntry(this);
		}

		// add new reference:
		if (this.event != null && !this.event.getEntries().contains(this)) {// if reference to this does not exist
			this.event.addEntry(this);
		}
	}

	public Set<ScoredRubric> getScoredRubrics() {
		return scoredRubrics;
	}

	@SuppressWarnings("unchecked")
	public void setScoredRubrics(Set<ScoredRubric> scoredRubrics) {
		// swap set of sub scores:
		Set<ScoredRubric> oldScoredRubrics = this.scoredRubrics;
		this.scoredRubrics = new HashSet<ScoredRubric>();
		if (scoredRubrics != null) {
			scoredRubrics = (Set<ScoredRubric>) ((HashSet<ScoredRubric>) scoredRubrics).clone();
		} else {
			scoredRubrics = new HashSet<ScoredRubric>();
		}

		// remove old references to this:
		for (ScoredRubric osr : oldScoredRubrics) {
			if (osr.getEntry() != null && osr.getEntry().equals(this)) {
				osr.setEntry(null);
			}
		}

		// set new scored rubrics:
		this.scoredRubrics = scoredRubrics;

		// add new references to this:
		for (ScoredRubric sr : this.scoredRubrics) {
			if (sr.getEntry() == null || !sr.getEntry().equals(this)) {
				sr.setEntry(this);
			}
		}
	}

	public boolean addScoredRubric(ScoredRubric sr) {
		// error check:
		if (scoredRubrics.contains(sr)) {// if duplicate exists
			// do not add:
			return false;
		}

		// add reference:
		boolean wasAdded = scoredRubrics.add(sr);

		// reciprocate reference:
		if (sr.getEntry() == null || !sr.getEntry().equals(this)) {// if reference to this exists
			sr.setEntry(this);
		}

		return wasAdded;
	}

	public boolean removeScoredRubric(ScoredRubric sr) {
		boolean wasRemoved = scoredRubrics.remove(sr);

		// remove reference to this:
		if (sr.getEntry() != null && sr.getEntry().equals(this)) {// if reference to this exists
			// remove reference:
			sr.setEntry(null);
		}

		return wasRemoved;
	}

	public String getTeamName() {
		return teamName;
	}

	public void setTeamName(String teamName) {
		this.teamName = teamName;
	}

	public String getTeamLeadName() {
		return teamLeadName;
	}

	public void setTeamLeadName(String teamLeadName) {
		this.teamLeadName = teamLeadName;
	}

	public String getTeamLeadEmail() {
		return teamLeadEmail;
	}

	public void setTeamLeadEmail(String teamLeadEmail) {
		this.teamLeadEmail = teamLeadEmail;
	}

	public String getTeamMemberNames() {
		return teamMemberNames;
	}

	public void setTeamMemberNames(String teamMemberNames) {
		this.teamMemberNames = teamMemberNames;
	}

	public int getGroupNumber() {
		return groupNumber;
	}

	/*
	 * Sets group number. Must be in the range 0 <= groupNumber < 5.
	 */
	public boolean setGroupNumer(int groupNumber) {
		// error check:
		if (groupNumber < 0 || groupNumber >= 5) {// if out of range
			return false;
		}

		// set new group number:
		this.groupNumber = groupNumber;
		return true;
	}

	@Override
	public String toString() {
		String output = "Entry [";
		output += "\n\tid=" + id;
		output += ",\n\ttitle=";
		output += (title != null) ? title : "null";
		output += ",\n\tdescription=";
		output += (description != null) ? description : "null";
		output += ",\n\tevent_id=";
		output += (event != null) ? event.getId() : "null";
		output += ",\n\tNumber of Scored Rubrics=" + scoredRubrics.size();
		output += "\n]";
		return output;
	}

}
