package com.evaluate.models;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;

@Entity
@Table(name="Event")
public class Event implements Serializable {

	private static final long serialVersionUID = 1L;
	
	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	@NotNull
	@Column(name = "id", nullable = false, unique = true)
	private long id;

	@Column(name = "name", nullable = false)
	private String name;
	
	@Column(name = "description")
	private String description;
	
	@Column(name = "date_time")
	private GregorianCalendar dateTime;
	
	@Column(name = "date")
	private String date;
	
	@Column(name = "time")
	private String time;

	@Column(name = "location")
	private String location;
	
	@OneToOne(mappedBy="event")
	private Chair chair;
	
	@OneToMany(mappedBy="event")
	private List<Judge> judges;
	
	@OneToOne
    @JoinColumn(name = "rubric_id")
	private Rubric rubric;
	
	@OneToMany(mappedBy="event")
	@JsonProperty(access = Access.WRITE_ONLY)
	private List<Entry> entries;
	
	@OneToMany(mappedBy="event")
	@JsonProperty(access = Access.WRITE_ONLY)
	private List<InviteCode> inviteCodes;
	
	@Column(name="invite_counter")
	@JsonProperty(access = Access.WRITE_ONLY)
	private int inviteCounter;
	
	/*
	 * Constructors:
	 */
	
	public Event() {
		super();
		setName(null);
		description = null;
		dateTime = null;
		location = null;
		chair = null;
		judges = new ArrayList<Judge>();
		rubric = null;
		entries = new ArrayList<Entry>();
		inviteCodes = new ArrayList<InviteCode>();
		inviteCounter = 0;
	}
	
	/*
	 * Getters and Setters:
	 */
	
	public long getId() {
		return id;
	}
	
	public void setId(long id) {
		this.id = id;
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		if ( name == null ) {
			this.name = "";
			return;
		}
		this.name = name;
	}
	
	public String getDescription() {
		return description;
	}
	
	public void setDescription(String description) {
		if ( description == null || description.isEmpty() ) {
			description = null;
		}
		this.description = description;
	}
	
	public GregorianCalendar getDateTime() {
		return dateTime;
	}

	public void setDateTime(GregorianCalendar dateTime) {
		this.dateTime = dateTime;
	}
	
	public String getDate() {
		return date;
	}

	public void setDate(String date) {
		this.date = date;
	}
	
	public String getTime() {
		return time;
	}

	public void setTime(String time) {
		this.time = time;
	}

	public String getLocation() {
		return location;
	}

	public void setLocation(String location) {
		this.location = location;
	}

	public Chair getChair() {
		return chair;
	}
	
	public void setChair(Chair chair) {
		//check if chair already referenced
		if ( this.chair != null && this.chair.equals(chair) ) {
			return;
		}
		
		Chair oldChair = this.chair;//save old chair
		this.chair = chair;
		if ( oldChair != null && this.equals(oldChair.getEvent()) ) {
			//remove reference to this:
			oldChair.setEvent(null);
		}
		
		//add new reference to this:
		if ( chair != null && !this.equals(chair.getEvent()) ) {
			//add reference:
			chair.setEvent(this);
		}
	}
	
	public List<Judge> getJudges() {
		return judges;
	}
	
	@SuppressWarnings("unchecked")
	public void setJudges(List<Judge> judges) {
		//swap lists:
		List<Judge> oldJudges = this.judges;
		this.judges = new ArrayList<Judge>();
		if ( judges != null ) {
			judges = (List<Judge>)((ArrayList<Judge>)judges).clone();
		} else {
			judges = new ArrayList<Judge>();
		}
		
		//remove old references to this:
		for (Judge oj : oldJudges) {
			if ( this.equals(oj.getEvent()) ) {
				oj.setEvent(null);
			}
		}
		
		//set new list:
		this.judges = judges;
		
		//add new references to this:
		for (Judge j : this.judges) {
			if ( j.getEvent() == null || !j.getEvent().equals(this) ) {
				j.setEvent(this);
			}
		}
	}
	
	/***
	 * Adds a judge to the list of judges and 
	 * adds the event reference to this instance if it 
	 * does not already exist. Returns true if the 
	 * judge is added successfully and is not a 
	 * duplicate of another judge already in the list. 
	 * 
	 * @param judge Judge to be added
	 * @return True if newly added to the list
	 */
	public boolean addJudge(Judge judge) {
		//error check:
		if ( judges.contains(judge) || judge == null ) {
			//do not add
			return false;
		}
		
		//add to list and save output to return at end:
		boolean output = judges.add(judge); 
		
		//reciprocate relation:
		if ( !this.equals(judge.getEvent()) ) {
			judge.setEvent(this);
		}
		
		return output;
	}
	
	public boolean removeJudge(Judge judge) {
		if (judge == null) {
			return false;
		}
		
		//remove relation:
		boolean output = judges.remove(judge);
		
		//remove reciprocal relation:
		if ( this.equals(judge.getEvent()) ) {
			judge.setEvent(null);
		}
		
		return output;
	}
	
	public Judge removeJudge(int index) {
		//remove relation:
		Judge output = judges.remove(index);
		
		//remove reciprocal relation:
		if ( this.equals(output.getEvent()) ) {
			output.setEvent(null);
		}
		
		return output;
	}
	
	public Rubric getRubric() {
		return rubric;
	}
	
	public void setRubric(Rubric rubric) {
		//check if rubric already referenced
		if ( this.rubric != null && this.rubric.equals(rubric) ) {
			return;
		}
		
		Rubric oldRubric = this.rubric;//save old rubric
		this.rubric = rubric;
		if ( oldRubric != null && this.equals(oldRubric.getEvent()) ) {
			//remove reference to this:
			oldRubric.setEvent(null);
		}
		
		//add new reference to this:
		if ( rubric != null && !this.equals(rubric.getEvent()) ) {
			//add reference:
			rubric.setEvent(this);
		}
	}
	
	public List<Entry> getEntries() {
		return entries;
	}
	
	@SuppressWarnings("unchecked")
	public void setEntries(List<Entry> entries) {
		//swap lists:
		List<Entry> oldEntries = this.entries;
		this.entries = new ArrayList<Entry>();
		if ( entries != null ) {
			entries = (List<Entry>)((ArrayList<Entry>)entries).clone();
		} else {
			entries = new ArrayList<Entry>();
		}
		
		//remove old references to this:
		for (Entry oj : oldEntries) {
			if ( this.equals(oj.getEvent()) ) {
				oj.setEvent(null);
			}
		}
		
		//set new list:
		this.entries = entries;
		
		//add new references to this:
		for (Entry j : this.entries) {
			if ( j.getEvent() == null || !j.getEvent().equals(this) ) {
				j.setEvent(this);
			}
		}
	}
	
	/***
	 * Adds a entry to the list of entries and 
	 * adds the event reference to this instance if it 
	 * does not already exist. Returns true if the 
	 * entry is added successfully and is not a 
	 * duplicate of another entry already in the list. 
	 * 
	 * @param entry Entry to be added
	 * @return True if newly added to the list
	 */
	public boolean addEntry(Entry entry) {
		//error check:
		if ( entries.contains(entry) || entry == null ) {
			//do not add
			return false;
		}
		
		//add to list and save output to return at end:
		boolean output = entries.add(entry); 
		
		//reciprocate relation:
		if ( !this.equals(entry.getEvent()) ) {
			entry.setEvent(this);
		}
		
		return output;
	}
	
	public boolean removeEntry(Entry entry) {
		if (entry == null) {
			return false;
		}
		
		//remove relation:
		boolean output = entries.remove(entry);
		
		//remove reciprocal relation:
		if ( this.equals(entry.getEvent()) ) {
			entry.setEvent(null);
		}
		
		return output;
	}
	
	public Entry removeEntry(int index) {
		//remove relation:
		Entry output = entries.remove(index);
		
		//remove reciprocal relation:
		if ( this.equals(output.getEvent()) ) {
			output.setEvent(null);
		}
		
		return output;
	}
	
	public List<InviteCode> getInviteCodes() {
		return inviteCodes;
	}
	
	@SuppressWarnings("unchecked")
	public void setInviteCodes(List<InviteCode> inviteCodes) {
		//swap lists:
		List<InviteCode> oldInviteCodes = this.inviteCodes;
		this.inviteCodes = new ArrayList<InviteCode>();
		if ( inviteCodes != null ) {
			inviteCodes = (List<InviteCode>)((ArrayList<InviteCode>)inviteCodes).clone();
		} else {
			inviteCodes = new ArrayList<InviteCode>();
		}
		
		//remove old references to this:
		for (InviteCode oic : oldInviteCodes) {
			if ( this.equals(oic.getEvent()) ) {
				oic.setEvent(null);
			}
		}
		
		//set new list:
		this.inviteCodes = inviteCodes;
		
		//add new references to this:
		for (InviteCode ic : this.inviteCodes) {
			if ( ic.getEvent() == null || !ic.getEvent().equals(this) ) {
				ic.setEvent(this);
			}
		}
	}
	
	public boolean addInviteCode(InviteCode inviteCode) {
		//error check:
		if ( inviteCodes.contains(inviteCode) || inviteCode == null ) {
			//do not add
			return false;
		}
		
		//add to list and save output to return at end:
		boolean output = inviteCodes.add(inviteCode); 
		
		//reciprocate relation:
		if ( !this.equals(inviteCode.getEvent()) ) {
			inviteCode.setEvent(this);
		}
		
		return output;
	}
	
	public boolean removeInviteCode(InviteCode inviteCode) {
		if (inviteCode == null) {
			return false;
		}
		
		//remove relation:
		boolean output = inviteCodes.remove(inviteCode);
		
		//remove reciprocal relation:
		if ( this.equals(inviteCode.getEvent()) ) {
			inviteCode.setEvent(null);
		}
		
		return output;
	}
	
	public InviteCode removeInviteCode(int index) {
		//remove relation:
		InviteCode output = inviteCodes.remove(index);
		
		//remove reciprocal relation:
		if ( this.equals(output.getEvent()) ) {
			output.setEvent(null);
		}
		
		return output;
	}
	
	public int getInviteCounter() {
		return inviteCounter;
	}

	/**
	 * The invite counter should only be modified by InviteCode since it is used
	 * during InviteCode generation.
	 * 
	 * @param inviteCounter new int value
	 */
	public void setInviteCounter(int inviteCounter) {
		this.inviteCounter = inviteCounter;
	}
	
	/**
	 * The invite counter should only be modified by InviteCode since it is used
	 * during InviteCode generation. This pre-increments the invite counter and
	 * returns its new value.
	 * 
	 * @return new invite counter value
	 */
	public int incInviteCounter() {
		if ( inviteCounter == 99 ) {
			inviteCounter = 0;
		}
		return ++inviteCounter;
	}

	/**
	 * This method converts the Calendar data type into two separate string so that
	 * the date and time do not have to be formatted in the front end.
	 */
	public void refreshDateTime() {
		Calendar temp = (Calendar) dateTime.clone();
		temp.clear(Calendar.ZONE_OFFSET);
		String date = temp.getDisplayName(Calendar.MONTH, Calendar.SHORT, Locale.US);
		date += " " + temp.get(Calendar.DAY_OF_MONTH) + ", ";
		date += temp.get(Calendar.YEAR);
		String time = temp.get(Calendar.HOUR) + ":";
		int mins = temp.get(Calendar.MINUTE);
		time += mins < 10 ? "0" : "";
		time += mins + " ";
		time += (temp.get(Calendar.AM_PM) == 1) ? "PM" : "AM";
		this.date = date;
		this.time = time;
	}

	@Override
	public String toString() {
		String output = "Event [";
		output += "\n\tid=" + id;
		output += "\n\tname=";
		output += (name != null) ? name : "null";
		output += "\n\tdescription=";
		output += (description != null) ? description : "null";
		output += "\n\tdate_time=";
		output += (dateTime != null) ? dateTime : "null";
		output += "\n\tchair_id=";
		output += (chair != null) ? chair.getId() : "null";
		output += "\n\tNumber of Judges=" + judges.size();
		output += "\n\trubric_id=";
		output += (rubric != null) ? rubric.getId() : "null";
		output += "\n\tNumber of Entries=" + entries.size();
		output += "\n]";
		return output;
	}
	
}
