package com.evaluate.models;

import java.io.Serializable;
import java.util.Random;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;

@Entity
@Table(name = "Invite_Code")
public class InviteCode implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@Column(name = "code", nullable = false, unique = true)
	private String code;

	@ManyToOne
	@NotNull
	@JoinColumn(name = "event_id", nullable = false)
	@JsonProperty(access = Access.WRITE_ONLY)
	private Event event;

	// CONSTRUCTORS

	public InviteCode() {
		code = null;
		event = null;
	}
	
	public InviteCode(Event event) {
		code = null;
		this.event = null;
		setEvent(event);
		generateCode();
	}

	// PUBLIC METHODS

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public Event getEvent() {
		return event;
	}

	public void setEvent(Event event) {
		// check if event already referenced
		if (this.event != null && this.event.equals(event)) {
			return;
		}

		Event oldEvent = this.event;// save old event
		this.event = event;
		if (oldEvent != null && oldEvent.getInviteCodes().contains(this)) {
			// remove reference to this:
			oldEvent.removeInviteCode(this);
		}

		// add new reference to this:
		if (event != null && !event.getInviteCodes().contains(this)) {
			// add reference:
			event.addInviteCode(this);
		}
	}

	/**
	 * Generates a pseudo random invite code which is stored in its code variable.
	 * Must have an associated event in order to generate a code successfully. There
	 * is no guarantee this code is unique. It must be saved in the repository
	 * successfully to gaurentee no matching codes exist
	 * 
	 * @return true if generated code, false otherwise
	 */
	public boolean generateCode() {
		// check if has event:
		if (event == null) {
			return false;
		}

		// modify combined length:
		long eventId = event.getId();
		int counter = event.incInviteCounter();
		int randomDigits = 0;
		String base10 = combine(eventId + "", counter + "", randomDigits);
		if (base10.length() > 18) {
			randomDigits = -(base10.length() - 18);
			base10 = combine(eventId + "", counter + "", randomDigits);
		}

		if (Long.parseLong(base10) > 308000000) {
			while (Long.parseLong(base10) > 308000000) {
				base10 = combine(eventId + "", counter + "", --randomDigits);
			}
		} else if (Long.parseLong(base10) < 500000) {
			while (Long.parseLong(base10) < 500000) {
				base10 = combine(eventId + "", counter + "", ++randomDigits);
			}
		}

		// convert to base26:
		String base26 = Long.toString(Long.parseLong(base10), 26).toUpperCase();

		// extend to 6 characters and swap bad characters:
		base26 = swap(base26);
		this.code = base26;

		return true;
	}

	public String toString() {
		String output = "InviteCode [";
		output += "\n\tcode=";
		output += (code != null) ? code : "null";
		output += "\n\tevent_id=";
		output += (event != null) ? event.getId() : "null";
		output += "\n]";
		return output;
	}

	// PRIVATE METHODS

	/**
	 * Combines the eventId and counter in the following way if randomDigits is
	 * nonnegative: counter + randomDigits + eventId. Otherwise, digits are removed
	 * from the most significant positions of eventId.
	 * 
	 * @param eventId
	 * @param counter
	 * @param randomDigits the number of random digits add
	 * @return combined string
	 */
	private String combine(String eventId, String counter, int randomDigits) {
		String output = counter;
		if (randomDigits >= 0) {
			Random r = new Random();
			for (int i = 0; i < randomDigits; i++) {
				output += r.nextInt(10);
			}
			output += eventId;
		} else {
			output += eventId.substring(-randomDigits);
		}
		return output;
	}

	private String swap(String code) {
		// swap characters:
		char[] chars = code.toCharArray();
		for (int i = 0; i < chars.length; i++) {
			switch (chars[i]) {
			case '0':
				chars[i] = 'Q';
				break;
			case 'O':
				chars[i] = 'R';
			}
		}
		code = new String(chars);

		// extend:
		if (code.length() == 5) {
			code = "Q" + code;
		}
		return code;
	}

}
