package com.evaluate.models;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import org.hibernate.annotations.DiscriminatorOptions;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonSubTypes.Type;
import com.fasterxml.jackson.annotation.JsonTypeInfo;

@Entity
@Inheritance
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.PROPERTY, property = "role_type")
@JsonSubTypes({ @Type(value = Judge.class, name = "J"), @Type(value = Chair.class, name = "C") })
@DiscriminatorColumn(name = "role_type")
@DiscriminatorOptions(force = true)
@Table(name = "Role")
public abstract class Role implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	@NotNull
	@Column(name = "id", nullable = false, unique = true)
	private long id;

	@ManyToOne
	@NotNull
	@JoinColumn(name = "account_id", nullable = false)
	@JsonProperty(access = Access.WRITE_ONLY)
	private Account account;

	@ManyToOne
	@NotNull
	@JoinColumn(name = "event_id", nullable = false)
	@JsonProperty(access = Access.WRITE_ONLY)
	protected Event event;

	@Column(name = "group_number")
	protected int groupNumber;

	@OneToMany(mappedBy = "judge")
	@JsonProperty(access = Access.WRITE_ONLY)
	private Set<ScoredRubric> scoredRubrics;

	public Role() {
		super();
		account = null;
		event = null;
		scoredRubrics = new HashSet<ScoredRubric>();
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public Account getAccount() {
		return account;
	}

	public void setAccount(Account account) {
		// error check:
		if (account != null && account.equals(this.account)) {// if duplicate
			// do nothing:
			return;
		}

		// swap:
		Account oldAccount = this.account;
		this.account = account;

		// remove old reference:
		if (oldAccount != null && oldAccount.getRoles().contains(this)) {// if reference to this exists
			oldAccount.removeRole(this);
		}

		// add new reference:
		if (this.account != null && !this.account.getRoles().contains(this)) {// if reference to this does not exist
			this.account.addRole(this);
		}
	}

	public Event getEvent() {
		return event;
	}

	public abstract void setEvent(Event event);

	public int getGroupNumber() {
		return groupNumber;
	}
	
	public abstract boolean setGroupNumber(int groupNumber);
	
	public Set<ScoredRubric> getScoredRubrics() {
		return scoredRubrics;
	}

	@SuppressWarnings("unchecked")
	public void setScoredRubric(Set<ScoredRubric> scoredRubrics) {
		// swapping old set
		Set<ScoredRubric> oldScoredRubrics = this.scoredRubrics;
		this.scoredRubrics = new HashSet<ScoredRubric>();
		if (scoredRubrics != null) {
			scoredRubrics = (Set<ScoredRubric>) ((HashSet<ScoredRubric>) scoredRubrics).clone();
			// cloning the copy
		} else {
			scoredRubrics = new HashSet<ScoredRubric>();
		}

		// removing old references to this
		for (ScoredRubric oldSR : oldScoredRubrics) {
			if (oldSR.getJudge() != null && oldSR.getJudge().equals(this)) {
				oldSR.setJudge(null);
			}
		}

		// set new scored rubric set:
		this.scoredRubrics = scoredRubrics;

		// adding new references to this
		for (ScoredRubric newSR : this.scoredRubrics) {
			if (newSR.getJudge() == null || !newSR.getJudge().equals(this)) {
				newSR.setJudge(this);
			}
		}
	}

	public boolean addScoredRubric(ScoredRubric addSR) {
		if (addSR == null) {
			return false;
		} else if (scoredRubrics.contains(addSR)) {// if duplicate exists: do not add
			return false;
		}

		// adding new references
		boolean wasAdded = scoredRubrics.add(addSR);

		// reciprocating reference
		if (addSR.getJudge() == null || !addSR.getJudge().equals(this)) {// if reference to this exists
			addSR.setJudge(this);
		}

		return wasAdded;
	}

	public boolean removeScoredRubric(ScoredRubric remSR) {
		// error check:
		if (remSR == null) {
			return false;
		}

		boolean wasRemoved = scoredRubrics.remove(remSR);

		// removing reference to this
		if (remSR.getJudge() != null && remSR.getJudge().equals(this)) {// if reference to this exists
			// remove reference by setting it to null
			remSR.setJudge(null);
		}

		return wasRemoved;
	}

	@Override
	public String toString() {
		String output = "";
		output += "\n\tid=" + id;
		output += ",\n\taccount_id=";
		output += (account != null) ? account.getId() : null;
		output += ",\n\tevent_id=";
		output += (event != null) ? event.getId() : null;
		output += "\n]";
		return output;
	}

}
