package com.evaluate.models;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;

@Entity
@Table(name="Rubric")
public class Rubric implements Serializable{
	
	private static final long serialVersionUID = 1L;
	
	@NotNull
	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	@Column(name="id", nullable = false, unique = true)
	private long id;
	
	@NotNull
	@OneToOne(mappedBy="rubric")
	@JsonProperty(access = Access.WRITE_ONLY)
	private Event event;

	@OneToMany(mappedBy = "rubric")
	private List<Category> categories;
	
	@OneToMany(mappedBy = "template")
	@JsonProperty(access = Access.WRITE_ONLY)
	private Set<ScoredRubric> scoredRubrics;

	@NotNull
	@Column(name= "total_max_score", nullable = false)
	private int totalMaxScore;

	public Rubric() {
		super();
		this.totalMaxScore = 0;
		this.event = null;
		this.categories = new ArrayList<Category>();
		this.scoredRubrics = new HashSet<ScoredRubric>();
	}
	
	public Rubric(List<Category> categories, Set<ScoredRubric> scoredRubrics) {
		super();
		this.totalMaxScore = 0;
		this.event = null;
		this.categories = new ArrayList<Category>();
		setCategories(categories);
		this.scoredRubrics = new HashSet<ScoredRubric>();
		setScoredRubrics(scoredRubrics);
	}
	
	public Long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}
	
	public Event getEvent() {
		return event;
	}
	
	public void setEvent(Event event) {
		//check if chair already referenced
		if ( this.event != null && this.event.equals(event) ) {
			return;
		}
		
		Event oldEvent = this.event;//save old event
		this.event = event;
		if ( oldEvent != null && this.equals(oldEvent.getRubric()) ) {
			//remove reference to this:
			oldEvent.setRubric(null);
		}
		
		//add new reference to this:
		if ( event != null && !this.equals(event.getRubric()) ) {
			//add reference:
			event.setRubric(this);
		}
	}
	
	public int getTotalMaxScore() {
		return totalMaxScore;
	}

	public void setTotalMaxScore(int totalMaxScore) {
		this.totalMaxScore = totalMaxScore;
	}
	
	public List<Category> getCategories() {
		return categories;
	}

	@SuppressWarnings("unchecked")
	public void setCategories(List<Category> categories) {
		//swapping list
		List<Category> oldCategories = this.categories;
		this.categories = new ArrayList<Category>();
		if ( categories != null ) {
			categories = (List<Category>)((ArrayList<Category>)categories).clone();
			// cloning the copy
		} else {
			categories = new ArrayList<Category>();
		}
		
		//removing old references to this
		for (Category oldCa : oldCategories) {
			if ( oldCa.getRubric() != null && oldCa.getRubric().equals(this) ) {
				oldCa.setRubric(null);
			}
		}
		
		//set new categories:
		this.categories = categories;
		
		//adding new references to this
		for (Category newCa : this.categories) {
			if ( newCa.getRubric() == null || !newCa.getRubric().equals(this) ) {
				newCa.setRubric(this);
			}
		}
	}
	
	public boolean addCategory(Category addCa) { 
		if ( addCa == null ) {
			return false;
		} else if ( categories.contains(addCa) ) {//if duplicate exists: do not add
			return false;
		}

		//adding new references
		boolean wasAdded = categories.add(addCa);

		//reciprocating reference
		if ( addCa.getRubric() == null || !addCa.getRubric().equals(this) ) {//if reference to this exists
			addCa.setRubric(this);
		}

		return wasAdded;
	}

	public Category removeCategory(int index) {
		if ( index < 0 || index >= categories.size() ) {//if index out of bounds
			return null;
		}
		return categories.remove(index);
	}
	
	public boolean removeCategory(Category remCa) {
		if ( remCa == null ) {
			return false;
		}
		
		boolean wasRemoved = categories.remove(remCa);
		
		//removing reference to this
		if ( remCa.getRubric() != null && remCa.getRubric().equals(this) ) {//if reference to this exists
			//remove reference by setting it to null
			remCa.setRubric(null);
		}
		return wasRemoved;
	}
	
	public Set<ScoredRubric> getScoredRubrics() {
		return scoredRubrics;
	}

	@SuppressWarnings("unchecked")
	public void setScoredRubrics(Set<ScoredRubric> scoredRubrics) {
		//swapping old set
		Set<ScoredRubric> oldScoredRubrics = this.scoredRubrics;
		this.scoredRubrics = new HashSet<ScoredRubric>();
		if ( scoredRubrics != null ) {
			scoredRubrics = (Set<ScoredRubric>)((HashSet<ScoredRubric>)scoredRubrics).clone();
			// cloning the copy 
		} else {
			scoredRubrics = new HashSet<ScoredRubric>();
		}
		
		//removing old references to this
		for (ScoredRubric oldSR : oldScoredRubrics) {
			if ( oldSR.getTemplate() != null && oldSR.getTemplate().equals(this) ) {
				oldSR.setTemplate(null);
			}
		}
		
		//set new scored rubric set:
		this.scoredRubrics = scoredRubrics;
		
		//adding new references to this
		for (ScoredRubric newSR : this.scoredRubrics) {
			if ( newSR.getTemplate() == null || !newSR.getTemplate().equals(this) ) {
				newSR.setTemplate(this);
			}
		}
	}
	
	public boolean addScoredRubric(ScoredRubric addSR) { 
		if ( addSR == null ) {
			return false;
		} else if ( scoredRubrics.contains(addSR) ) {//if duplicate exists: do not add
			return false;
		}
		
		//adding new references
		boolean wasAdded = scoredRubrics.add(addSR);
		
		//reciprocating reference
		if ( addSR.getTemplate() == null || !addSR.getTemplate().equals(this) ) {//if reference to this exists
			addSR.setTemplate(this);
		}
		
		return wasAdded;
	}
	
	public boolean removeScoredRubric(ScoredRubric remSR) {
		//error check:
		if ( remSR == null ) {
			return false;
		}
		
		boolean wasRemoved = scoredRubrics.remove(remSR);
		
		//removing reference to this
		if ( remSR.getTemplate() != null && remSR.getTemplate().equals(this) ) {//if reference to this exists
			//remove reference by setting it to null
			remSR.setTemplate(null);
		}
		
		return wasRemoved;
	}
	
	@Override
	public String toString() {
		String output = "Rubric [";
		output += "\n\tid=" + id;
		output += ",\n\tevent_id=";
		output += (event != null) ? event.getId() : "null";
		output += ",\n\ttotal_max_score=" + totalMaxScore;
		output += ",\n\tNumber of Categories=" + categories.size();
		output += ",\n\tNumber of Scored Rubrics=" + scoredRubrics.size();
		output += "\n]";
		return output;
	}
	
	public void recalculateTotalMaxScore() {
 		if ( categories.size() == 0 ) {//if no sub scores
 			//do nothing, because pointsAwarded is not generated:
 			return;
 		}

 		int tms = 0;
 		for (Category ca : categories) {
 			ca.recalculateMaxScore();
 			tms += ca.getMaxScore();
 		}
 		this.totalMaxScore = tms;
 	}

}
