package com.evaluate.models;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;

@Entity
@Table(name="Score")
public class Score implements Serializable {
	
	private static final long serialVersionUID = 1L;

	@Id
	@NotNull
	@GeneratedValue(strategy=GenerationType.SEQUENCE)
	@Column(name="id", nullable=false, unique=true)
	private long id;

	@NotNull
	@ManyToOne
	@JoinColumn(name="template_id", nullable=false)
	private Category template;
	
	@NotNull
	@Column(name="points_awarded", nullable=false)
	private int pointsAwarded;
	
	@ManyToOne
	@JoinColumn(name="parent_score_id")
	@JsonProperty(access = Access.WRITE_ONLY)
	private Score parentScore;
	
	@OneToMany(mappedBy="parentScore")
	private List<Score> subScores;
	
	@ManyToOne
	@JoinColumn(name="scored_rubric_id")
	@JsonProperty(access = Access.WRITE_ONLY)
	private ScoredRubric scoredRubric;

	//Tracks if a score below it has changed
	@Transient
	private boolean isUpToDate;
	
	/*
	 * Constructors:
	 */

	public Score() {
		super();
		this.template = null;
		this.pointsAwarded = 0;
		this.parentScore = null;
		this.subScores = new ArrayList<Score>();
		this.scoredRubric = null;
		this.isUpToDate = true;
	}

	public Score(Category template) {
		super();
		setTemplate(template);
		this.subScores = new ArrayList<Score>();
		setPointsAwarded(0);
		this.parentScore = null;
		this.scoredRubric = null;
	}

	public Score(Category template, int pointsAwarded) {
		super();
		setTemplate(template);
		this.subScores = new ArrayList<Score>();
		setPointsAwarded(pointsAwarded);
		this.parentScore = null;
		this.scoredRubric = null;
	}

	public Score(Category template, int pointsAwarded, Score parentScore) {
		super();
		setTemplate(template);
		this.subScores = new ArrayList<Score>();
		setPointsAwarded(pointsAwarded);
		setParentScore(parentScore);
		this.scoredRubric = null;
	}

	public Score(Category template, int pointsAwarded, ScoredRubric scoredRubric) {
		super();
		setTemplate(template);
		this.subScores = new ArrayList<Score>();
		setPointsAwarded(pointsAwarded);
		this.parentScore = null;
		this.setScoredRubric(scoredRubric);
	}

	public Score(Category template, Score parentScore, List<Score> subScores) {
		super();
		setTemplate(template);
		this.subScores = new ArrayList<Score>();
		setSubScores(subScores);
		setParentScore(parentScore);
		this.scoredRubric = null;
	}

	public Score(Category template, ScoredRubric scoredRubric, List<Score> subScores) {
		super();
		setTemplate(template);
		this.subScores = new ArrayList<Score>();
		setSubScores(subScores);
		this.parentScore = null;
		this.setScoredRubric(scoredRubric);
	}
	
	/*
	 * Getters and Setters:
	 */

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}
	
	public Category getTemplate() {
		return template;
	}
	
	public void setTemplate(Category template) {
		//error check:
		if ( this.template != null && this.template.equals(template) ) {//if resetting template 
			//do nothing:
			return;
		}
		
		Category oldTemplate = this.template;//save old template
		this.template = template;
		if ( oldTemplate != null && oldTemplate.getScores().contains(this) ) {//if reference to this exists
			//remove reference:
			oldTemplate.removeScore(this);
		}
		
		//add new reference:
		if ( template != null && !template.getScores().contains(this) ) {//if new reference doesn't exists 
			//add reference:
			template.addScore(this);
		}
	}

	public int getPointsAwarded() {
		return pointsAwarded;
	}

	/**
	 * Sets the points awarded for a score. Can 
	 * only be done for leaves. Must follow 
	 * 0 <= pointsAwarded <= template.maxScore
	 * 
	 * @param pointsAwarded Integer value
	 * @return true if successfully changed
	 */
	public boolean setPointsAwarded(int pointsAwarded) {
		//error check:
		if( subScores != null && subScores.size() != 0 ) {//if not a leaf
			return false;
		} else if ( template == null ) {//if no template 
			return false;
		} else if ( pointsAwarded < 0 || pointsAwarded > template.getMaxScore() ) {//if value out of bounds
			return false;
		} else if( this.pointsAwarded == pointsAwarded ) {//if no change in value:
			return false;
		}
		
		//set new score and recalculate:
		this.pointsAwarded = pointsAwarded;
		recalculateAll();
		return true;
	}
	
	public Score getParentScore() {
		return parentScore;
	}
	
	public void setParentScore(Score parentScore) {
		//error check:
		if ( this.parentScore != null && this.parentScore.equals(parentScore) ) {//if resetting parentScore
			//do nothing:
			return;
		}
		
		Score oldParent = this.parentScore;//save old parent
		this.parentScore = parentScore;
		if ( oldParent != null && oldParent.getSubScores().contains(this) ) {//if reference to this exists
			//remove reference:
			oldParent.removeSubScore(this);
		}
		
		//add new reference:
		if ( parentScore != null && !parentScore.getSubScores().contains(this) ) {//if new reference doesn't exists 
			//add reference:
			parentScore.addSubScore(this);
		}
	}
	
	public List<Score> getSubScores() {
		return subScores;
	}
	
	@SuppressWarnings("unchecked")
	public void setSubScores(List<Score> subScores) {
		//swap set of sub scores:
		List<Score> oldSubScores = this.subScores;
		this.subScores = new ArrayList<Score>();
		if ( subScores != null ) {
			subScores = (List<Score>)((ArrayList<Score>)subScores).clone();
		} else {
			subScores = new ArrayList<Score>();
		}
		
		//remove old references to this:
		for (Score oss : oldSubScores) {
			if ( this.equals(oss.parentScore) ) {
				oss.parentScore = null;
			}
		}
		
		//set new sub scores:
		this.subScores = subScores;
		
		//add new references to this:
		for (Score ss : this.subScores) {
			if ( ss.parentScore == null || !ss.parentScore.equals(this) ) {
				ss.setParentScore(this);
			}
		}
		
		//recalculate points awarded:
		recalculateAll();
	}
	
	/***
	 * Adds a sub score to the list of sub scores and 
	 * adds the parent reference to this class if it 
	 * does not already exist. Returns true if the 
	 * sub score is added successfully and is not a 
	 * duplicate of another sub score. 
	 * 
	 * @param s Sub score to be added
	 * @return True if added to the list
	 */
	public boolean addSubScore(Score s) {
		if ( subScores.contains(s) ) {//if duplicate
			//do not add:
			return false;
		}
		
		//add to list and save output to return at end:
		boolean output = subScores.add(s);
		
		//reciprocate relation:
		if ( s.getParentScore() == null || !s.getParentScore().equals(this) ) {//if reference isn't already reciprocated
			s.setParentScore(this);
		}
		
		//recalculate points awarded:
		recalculateAll();		
		return output;
	}
	
	public Score removeSubScore(int index) {
		//remove relation:
		Score scoreRemoved = subScores.remove(index);
		
		//remove reciprocal relation:
		if ( scoreRemoved.getParentScore() != null && scoreRemoved.getParentScore().equals(this) ) {//if reciprocal relation exists
			scoreRemoved.setParentScore(null);
		}
		
		//return removed:
		return scoreRemoved;
	}
	
	public boolean removeSubScore(Score s) {
		//remove relation:
		boolean wasRemoved = subScores.remove(s);
		
		//remove reciprocal relation:
		if ( s.getParentScore() != null && s.getParentScore().equals(this) ) {//if reciprocal relation exists
			s.setParentScore(null);
		}
		
		//return if was removed
		return wasRemoved;
	}
	
	public ScoredRubric getScoredRubric() {
		return scoredRubric;
	}
	
	public void setScoredRubric(ScoredRubric scoredRubric) {
		//error check:
		if ( this.scoredRubric != null && this.scoredRubric.equals(scoredRubric) ) {//if resetting scoredRubric
			//do nothing:
			return;
		}
		
		ScoredRubric oldScoredRubric = this.scoredRubric;
		this.scoredRubric = scoredRubric;
		if ( oldScoredRubric != null && oldScoredRubric.getScoredCategories().contains(this) ) {//if reference to this exists
			//remove reference:
			oldScoredRubric.removeScoredCategories(this);
		}
		
		//add new reference:
		if ( scoredRubric != null && !scoredRubric.getScoredCategories().contains(this) ) {
			scoredRubric.addScoredCategory(this);
		}
		
		recalculateAll();
	}

	/*
	 * Logic
	 */
	
	/**
	 * Recursively recalculates sub scores if their 
	 * associated points awarded have changed. But, does 
	 * not change the value of any parent nodes. 
	 */
	public void recalculatePointsAwarded() {
		if ( isUpToDate ) {//if already up to date
			//do nothing:
			return;
		}
		if ( subScores.size() == 0 ) {//if no sub scores
			//do nothing, because pointsAwarded is not generated:
			return;
		}
		
		int pa = 0;
		for (Score ss : subScores) {
			ss.recalculatePointsAwarded();
			pa += ss.pointsAwarded;
		}
		this.pointsAwarded = pa;
		isUpToDate = true;
	}

	@Override
	public String toString() {
		String output = "Score [";
		output += "\n\tid=" + id;
		output += ",\n\ttemplate_id=";
		output += (template != null) ? template.getId() : "null";
		output += ",\n\tpoints_awarded=" + pointsAwarded;
		output += ",\n\tparent_score_id=";
		output += (parentScore != null) ? parentScore.getId() : "=null";
		output += ",\n\tNumber of Subscores=" + subScores.size();
		output += ",\n\tscored_rubric_id=";
		output += (scoredRubric != null) ? scoredRubric.getId() : "null";
		output += "\n]";
		return output;
	}

	/*
	 * Private methods
	 */
	
	/***
	 * Recalculates all related points awarded which 
	 * have changed, including sub scores, parent scores, 
	 * and scored rubrics. This is done recursively. 
	 */
	private void recalculateAll() {
		isUpToDate = false;
		if ( parentScore != null ) {//if has parent
			//go up a level:
			parentScore.recalculateAll();
		} else if ( scoredRubric != null ) {//if has scored rubric
			//recalculate from top:
			scoredRubric.recalculateTotalScore();
		} else {//if has neither
			//recalculate from top:
			recalculatePointsAwarded();
		}
	}
	
}
