package com.evaluate.models;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

@Entity
@Table(name = "Scored_Rubric")
public class ScoredRubric {
	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	@NotNull
	@Column(name = "id", nullable = false, unique = true)
	private long id;

	@ManyToOne
	@NotNull
	@JoinColumn(name = "entry_id", nullable = false)
	private Entry entry;

	@ManyToOne
	@NotNull
	@JoinColumn(name = "template_id", nullable = false)
	private Rubric template;

	@ManyToOne
	@NotNull
	@JoinColumn(name = "judge_id", nullable = false)
	private Role judge;

	@Column(name = "total_score")
	private int totalScore;

	@OneToMany(mappedBy = "scoredRubric")
	private List<Score> scoredCategories;

	@Column(name = "judge_note", nullable = true)
	private String judgeNote;

	public ScoredRubric() {
		super();
		entry = null;
		template = null;
		judge = null;
		totalScore = 0;
		scoredCategories = new ArrayList<Score>();
		judgeNote = null;
	}

	public ScoredRubric(Entry entry, Rubric template) {
		super();
		setEntry(entry);
		setTemplate(template);
		judge = null;
		totalScore = 0;
		scoredCategories = new ArrayList<Score>();
		judgeNote = null;
	}

	public ScoredRubric(Entry entry, Rubric template, Role judge) {
		super();
		setEntry(entry);
		setTemplate(template);
		setJudge(judge);
		this.totalScore = 0;
		scoredCategories = new ArrayList<Score>();
		judgeNote = null;
	}

	public ScoredRubric(Entry entry, Rubric template, List<Score> scoredCategories, String judgeNote) {
		super();
		setEntry(entry);
		setTemplate(template);
		judge = null;
		this.scoredCategories = new ArrayList<Score>();
		setScoredCategories(scoredCategories);
		totalScore = 0;
		setJudgeNote(judgeNote);
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public Entry getEntry() {
		return entry;
	}

	public void setEntry(Entry entry) {
		// error check:
		if (this.entry != null && this.entry.equals(entry)) {// if resetting entry
			// do nothing:
			return;
		}
		Entry oldentry = this.entry;// save old entry
		this.entry = entry;
		if (oldentry != null && oldentry.getScoredRubrics().contains(this)) {// if reference to this exists
			// remove reference:
			oldentry.removeScoredRubric(this);
		}
		// add new reference:
		if (entry != null && !entry.getScoredRubrics().contains(this)) {// if new reference doesn't exists
			// add reference:
			entry.addScoredRubric(this);
		}
	}

	public Rubric getTemplate() {
		return template;
	}

	public void setTemplate(Rubric template) {

		if (this.template != null && this.template.equals(template)) {
			return;
		}

		Rubric oldTemplate = this.template;// save old template
		this.template = template;
		if (oldTemplate != null && oldTemplate.getScoredRubrics().contains(this)) {// if reference to this exists
			// remove reference:
			oldTemplate.removeScoredRubric(this);
		}

		// add new reference:
		if (template != null && !template.getScoredRubrics().contains(this)) {// if new reference doesn't exists
			// add reference:
			template.addScoredRubric(this);
		}
	}

	public Role getJudge() {
		return judge;
	}

	public void setJudge(Role judge) {
		if (this.judge != null && this.judge.equals(judge)) {
			return;
		}

		Role oldJudge = this.judge;// save old judge
		this.judge = judge;
		if (oldJudge != null && oldJudge.getScoredRubrics().contains(this)) {// if reference to this exists
			// remove reference:
			oldJudge.removeScoredRubric(this);
		}

		// add new reference:
		if (judge != null && !judge.getScoredRubrics().contains(this)) {// if new reference doesn't exists
			// add reference:
			judge.addScoredRubric(this);
		}
	}

	public int getTotalScore() {
		recalculateTotalScore();
		return totalScore;
	}

	/**
	 * You should avoid using this function because it has no effect on the
	 * categories below it. If you would like to alter the value of totalScore,
	 * change the values of the category tree. This will alter the value of
	 * totalScore to match.
	 * 
	 * @param totalScore
	 */
	public void setTotalScore(int totalScore) {
		if (template == null) {// if no template
			return;
		}
		if (totalScore < 0 || totalScore > template.getTotalMaxScore()) {// if out of bounds
			return;
		}
		this.totalScore = totalScore;
	}

	public List<Score> getScoredCategories() {
		return scoredCategories;
	}

	@SuppressWarnings("unchecked")
	public void setScoredCategories(List<Score> scoredCategories) {
		List<Score> oldScoredCategories = this.scoredCategories;
		this.scoredCategories = new ArrayList<Score>();
		if (scoredCategories != null) {
			scoredCategories = (List<Score>) ((ArrayList<Score>) scoredCategories).clone();
		} else {
			scoredCategories = new ArrayList<Score>();
		}

		// remove old references to this:
		for (Score osc : oldScoredCategories) {
			if (osc.getScoredRubric().equals(this)) {
				osc.setScoredRubric(null);
			}
		}

		// set new scored categories:
		this.scoredCategories = scoredCategories;

		// add new references to this:
		for (Score sc : this.scoredCategories) {
			if (sc.getScoredRubric() == null || !sc.getScoredRubric().equals(this)) {
				sc.setScoredRubric(this);
			}
		}

		recalculateTotalScore();
	}

	public boolean addScoredCategory(Score s) {
		if (s == null) {
			return false;
		} else if (scoredCategories.contains(s)) {// if duplicate exists: do not add
			return false;
		}

		// adding new references
		boolean wasAdded = scoredCategories.add(s);

		// reciprocating reference
		if (s.getScoredRubric() == null || !s.getScoredRubric().equals(this)) {// if reference to this exists
			s.setScoredRubric(this);
			;
		}

		return wasAdded;
	}

	public Score removeScoredCategory(int index) {
		if (index < 0 || index >= scoredCategories.size()) {// if index out of bounds
			return null;
		}
		return scoredCategories.remove(index);
	}

	public boolean removeScoredCategories(Score s) {
		if (s == null) {
			return false;
		}

		boolean wasRemoved = scoredCategories.remove(s);

		// removing reference to this
		if (s.getScoredRubric() != null && s.getScoredRubric().equals(this)) {// if reference to this exists
			// remove reference by setting it to null
			s.setScoredRubric(null);
		}
		return wasRemoved;
	}

	public String getJudgeNote() {
		return judgeNote;
	}

	public void setJudgeNote(String judgeNote) {
		if (judgeNote != null && judgeNote.equals("")) {
			this.judgeNote = null;
			return;
		} else if (this.judgeNote != null && this.judgeNote.equals(judgeNote)) {
			return;
		}
		this.judgeNote = judgeNote;
	}

	public void recalculateTotalScore() {
		if (scoredCategories.size() == 0) {// if no sub scores
			// do nothing, because pointsAwarded is not generated:
			return;
		}
		int ts = 0;
		for (Score ss : scoredCategories) {
			ss.recalculatePointsAwarded();
			ts += ss.getPointsAwarded();
		}
		this.totalScore = ts;
	}

	@Override
	public String toString() {
		String output = "ScoredRubric [";
		output += "\n\tid =" + id;
		output += "\n\tentry_id=";
		output += (entry != null) ? entry.getId() : "\n\tentry_id=null";
		output += "\n\ttemplate_id=";
		output += (template != null) ? template.getId() : "\n\ttemplate_id=null";
		output += "\n\ttotal_score=" + totalScore;
		output += "\n\tNumber of Scored Categories=" + scoredCategories.size();
		output += "\n\tjudge_note=";
		output += (judgeNote != null) ? judgeNote : "\n\tjudge_note=null";
		output += "\n]";
		return output;
	}

}
