package com.evaluate.services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import com.evaluate.models.*;
import com.evaluate.repositories.*;

@Service
public class AssignGroupService {

	@Autowired
	EntryRepository entryRepository;
	@Autowired
	RoleRepository roleRepository;
	@Autowired
	EventRepository eventRepository;
	@Autowired
	ScoredRubricRepository scoredRubricRepository;
	@Autowired
	ScoreRepository scoreRepository;

	public Map<String, Object> getInfo(long id) {
		// get event:
		Optional<Event> optEvent = eventRepository.findById(id);
		if (optEvent.isEmpty()) {
			return null;
		}
		Event event = optEvent.get();

		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Chair chair = event.getChair();
		if (user.getId() != chair.getAccount().getId()) {// if user is the chair
			return null;
		}

		// fetch list of entries:
		List<Entry> entries = event.getEntries();

		// fetch list of judges:
		List<Role> judges = new ArrayList<Role>(event.getJudges());
		judges.add(0, event.getChair());

		// fetch list of judge names:
		List<String> judgeNames = new ArrayList<String>();
		for (Role judge : judges) {
			judgeNames.add(judge.getAccount().getFullName());
		}

		// Put into map:
		Map<String, Object> map = new HashMap<>();
		map.put("entries", entries);
		map.put("judges", judges);
		map.put("judgeNames", judgeNames);

		return map;
	}

	public boolean deleteEntry(long eventId, long entryId) {
		// get event:
		Optional<Event> optEvent = eventRepository.findById(eventId);
		if (optEvent.isEmpty()) {
			return false;
		}
		Event event = optEvent.get();

		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Chair chair = event.getChair();
		if (user.getId() != chair.getAccount().getId()) {// if user is not the chair
			return false;
		}

		// check if entry is for event:
		Optional<Entry> optEntry = entryRepository.findById(entryId);
		if (optEntry.isEmpty() || optEntry.get().getEvent().getId() != event.getId()) {
			return false;
		}

		// delete:
		entryRepository.deleteById(entryId);
		return true;
	}

	public HttpStatus deleteJudge(long eventId, long judgeId) {
		// get event
		Optional<Event> optEvent = eventRepository.findById(eventId);
		if (optEvent.isEmpty()) {
			return HttpStatus.NOT_FOUND;
		}
		Event event = optEvent.get();

		// check authorization for event:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Chair chair = event.getChair();
		if (user.getId() != chair.getAccount().getId()) {// if user is not the chair
			return HttpStatus.NOT_FOUND;
		}

		// check authorization for judge:
		Optional<Role> optJudge = roleRepository.findById(judgeId);
		if (optJudge.isEmpty()// if judge does not exist
				|| optJudge.get().getEvent().getId() != event.getId()) {// if judge is not for event
			return HttpStatus.NOT_FOUND;
		}

		// check chair isn't being removed:
		if (optJudge.get().getId() == chair.getId()) {
			return HttpStatus.BAD_REQUEST;
		}

		// delete:
		this.deleteJudgeData(optJudge.get());
		return HttpStatus.OK;
	}

	public boolean save(long eventId, List<Entry> entries, List<Role> judges) {
		// get event:
		Optional<Event> optEvent = eventRepository.findById(eventId);
		if (optEvent.isEmpty()) {
			return false;
		}
		Event event = optEvent.get();

		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Chair chairFetch = event.getChair();
		if (user.getId() != chairFetch.getAccount().getId()) {// if user is the chair
			return false;
		}

		// Save Entries:
		for (Entry e : entries) {
			Optional<Entry> optEntry = entryRepository.findById(e.getId());
			if (optEntry.isEmpty()) {
				return false;
			}
			Entry entry = optEntry.get();
			if (entry.getEvent().getId() != event.getId()) {
				return false;
			}
			entry.setGroupNumer(e.getGroupNumber());
			entryRepository.save(entry);
		}

		// Save judges:
		for (int i = 0; i < judges.size(); i++) {
			Role oldJudge = judges.get(i);
			Optional<Role> optJudge = roleRepository.findById(oldJudge.getId());
			if (optJudge.isEmpty()) {
				return false;
			}
			Role judge = optJudge.get();
			if (judge.getEvent().getId() != event.getId()) {
				return false;
			}
			judge.setGroupNumber(oldJudge.getGroupNumber());
			roleRepository.save(judge);
		}

		return true;
	}

	private void deleteJudgeData(Role judge) {
		// delete scored rubrics:
		Set<ScoredRubric> scoredRubrics = judge.getScoredRubrics();
		for (ScoredRubric sr : scoredRubrics) {
			this.deleteScoredRubricData(sr);
		}

		// delete judge:
		roleRepository.delete(judge);
	}

	private void deleteScoredRubricData(ScoredRubric scoredRubric) {
		// delete scored categories:
		List<Score> scores = scoredRubric.getScoredCategories();
		for (Score s : scores) {
			this.deleteScoreData(s);
		}

		// delete scored rubric:
		scoredRubricRepository.delete(scoredRubric);
	}

	private void deleteScoreData(Score score) {
		// delete sub scores:
		List<Score> subScores = score.getSubScores();
		for (Score ss : subScores) {
			this.deleteScoreData(ss);
		}

		// delete score:
		scoreRepository.delete(score);
	}

}
