package com.evaluate.services;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Optional;

import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import com.evaluate.models.Account;
import com.evaluate.repositories.AccountRepository;

@Service
public class CreateAccountService {

	@Autowired
	private AccountRepository accountRepository;

	@Autowired
	private HashService hashService;

	@Autowired
	private MailService mailService;

	public ResponseEntity<?> saveAccount(Account newAccount) throws URISyntaxException {
		// check for existing account with given email:
		Optional<Account> optAccount = accountRepository.findByEmail(newAccount.getEmail());
		if (optAccount.isPresent()) {
			return new ResponseEntity<>(HttpStatus.CONFLICT);
		}

		// validate password requirements:
		String password = newAccount.getEncryptedPassword();
		if (password.length() < 6 || password.length() > 25) {
			return new ResponseEntity<>(HttpStatus.FORBIDDEN);
		}

		// encrypt password:
		newAccount.setEncryptedPassword(hashService.hash(newAccount.getEmail(), newAccount.getEncryptedPassword()));

		// save account:
		Account account = accountRepository.save(newAccount);

		// send confirmation email:
		String header = "e-Valuate Account Created";
		String body = "Hello " + account.getFullName() + ",\n\n";
		body += "This is simply a message confirming your account creation and welcoming you to e-Valuate.net!";
		mailService.sendEmail(account.getEmail(), header, body);

		return ResponseEntity.created(new URI("/register/account/" + account.getId())).body(account);
	}

	public ResponseEntity<?> getAccount() {
		// get account:
		Account account = (Account) SecurityUtils.getSubject().getPrincipal();
		account = accountRepository.findById(account.getId()).get();
		return ResponseEntity.ok(account);
	}

	public ResponseEntity<?> updateAccount(Account newAccount) {
		// get old account:
		Account oldAccount = (Account) SecurityUtils.getSubject().getPrincipal();
		oldAccount = accountRepository.findById(oldAccount.getId()).get();

		// verify password:
		String encryptedPassword = hashService.hash(oldAccount.getEmail(), newAccount.getEncryptedPassword());
		if (!encryptedPassword.equals(oldAccount.getEncryptedPassword())) {// invalid password
			return new ResponseEntity<>(HttpStatus.FORBIDDEN);
		}

		// check if email changed:
		if (!newAccount.getEmail().contentEquals(oldAccount.getEmail())) {
			// check that new email is free:
			Optional<Account> optEmail = accountRepository.findByEmail(newAccount.getEmail());
			if (optEmail.isPresent()) {
				return new ResponseEntity<>(HttpStatus.CONFLICT);
			}
			
			// encrypt was password with new email:
			oldAccount.setEmail(newAccount.getEmail());
			oldAccount.setEncryptedPassword(hashService.hash(newAccount.getEmail(), newAccount.getEncryptedPassword()));
		}

		// update old account:
		oldAccount.setFirstName(newAccount.getFirstName());
		oldAccount.setLastName(newAccount.getLastName());

		// save account:
		newAccount = accountRepository.save(oldAccount);

		// send confirmation email:
		String header = "e-Valuate Account Modified";
		String body = "Hello " + newAccount.getFullName() + ",\n\n";
		body += "This is simply a notification that some of your account information has changed.\n\n";
		body += "If you did not make these changes, please reset your password immediately by following the link:\n";
		body += MailService.domain + "forgotpassword";
		mailService.sendEmail(newAccount.getEmail(), header, body);

		return ResponseEntity.ok(newAccount);
	}

}
