package com.evaluate.services;

import java.util.Optional;

import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.evaluate.models.Account;
import com.evaluate.models.Chair;
import com.evaluate.models.Event;
import com.evaluate.repositories.EventRepository;
import com.evaluate.repositories.RoleRepository;

@Service
public class CreateEventService {

	@Autowired
	public EventRepository eventRepository;

	@Autowired
	public RoleRepository roleRepository;

	/**
	 * Returns an event with the given name.
	 * 
	 * @param id of event to find
	 * @return Optional<Event> with matching name
	 */
	public Optional<Event> getEvent(long id) {
		Optional<Event> optEvent = eventRepository.findById(id);
		if (optEvent.isEmpty()) {
			return Optional.empty();
		}
		Event event = optEvent.get();

		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Chair chair = event.getChair();
		if (user.getId() == chair.getAccount().getId()) {// if user is the chair
			return optEvent;
		}

		return Optional.empty();
	}

	/**
	 * Creates a new event in the database as well as a new chair role linking the
	 * account to the event.
	 * 
	 * @param event to be added
	 * @return saved instance of event for further operation.
	 */
	public Event createEvent(Event event) {
		// save new event:
		event.refreshDateTime();
		event = eventRepository.save(event);

		// create role:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Chair newChair = new Chair();
		newChair.setAccount(user);
		newChair.setEvent(event);
		roleRepository.save(newChair);

		return event;
	}

	/**
	 * Edits an existing event in the database. Leaves rubrics and roles alone.
	 * 
	 * @param newEvent to be modified
	 * @return saved instance of event for further operation
	 */
	public Event editEvent(Event newEvent) {
		// find event to modify:
		Optional<Event> optEvent = eventRepository.findById(newEvent.getId());
		if (optEvent.isEmpty()) {
			return null;
		}
		Event event = optEvent.get();
		
		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Chair chair = event.getChair();
		if (user.getId() != chair.getAccount().getId()) {// if user is NOT the chair
			return null;
		}
		
		// modify event:
		event.setName(newEvent.getName());
		event.setDateTime(newEvent.getDateTime());
		event.refreshDateTime();
		event.setLocation(newEvent.getLocation());
		event.setDescription(newEvent.getDescription());
		event = eventRepository.save(event);

		return event;
	}

}
