package com.evaluate.services;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Optional;

import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import com.evaluate.models.*;
import com.evaluate.repositories.EventRepository;
import com.evaluate.repositories.CategoryRepository;
import com.evaluate.repositories.RubricRepository;

@Service
public class CreateRubricService {

	private ResponseEntity<?> notFoundRE = new ResponseEntity<>(HttpStatus.NOT_FOUND);

	@Autowired
	private CategoryRepository categoryRepository;

	@Autowired
	private EventRepository eventRepository;

	@Autowired
	private RubricRepository rubricRepository;

	/**
	 * Returns the rubric for the event with the given id.
	 * 
	 * @param eventId of the rubric's event
	 * @return Optional<Rubric>
	 */
	public Optional<Rubric> getRubricByEvent(long eventId) {
		// get event:
		Optional<Event> optEvent = eventRepository.findById(eventId);
		if (optEvent.isEmpty()) {
			return Optional.empty();
		}

		// get rubric:
		if (optEvent.get().getRubric() == null) {
			return Optional.empty();
		}
		Rubric rubric = optEvent.get().getRubric();

		// check access:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		try {
			long chairAccountId = optEvent.get().getChair().getAccount().getId();
			if (user.getId() != chairAccountId) {
				return Optional.empty();
			}
		} catch (Exception e) {
			return Optional.empty();
		}

		return Optional.of(rubric);
	}

	/***
	 * Does the necessary error checking to see if the rubric can be saved. If it
	 * cannot be saved, the appropriate response entity is returned instead.
	 * Otherwise, the rubric is created.
	 * 
	 * @param eventId id of the event to set with the rubric
	 * @param rubric  to save in the database
	 * @return URI of the new rubric or an HTTP status
	 * @throws URISyntaxException
	 */
	public ResponseEntity<?> createRubricByEvent(long eventId, Rubric rubric) throws URISyntaxException {
		// find event:
		Optional<Event> optEvent = eventRepository.findById(eventId);
		if (optEvent.isEmpty()) {
			return notFoundRE;
		}
		Event event = optEvent.get();

		// check access:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		long chairAccountId = event.getChair().getAccount().getId();
		if (user.getId() != chairAccountId) {
			return notFoundRE;
		}

		// check for previous rubric:
		if (event.getRubric() != null) {// if has previous rubric
			return new ResponseEntity<>(HttpStatus.ALREADY_REPORTED);
		}

		// save the rubric:
		rubric.setEvent(event);
		rubric = this.saveRubric(rubric);
		return ResponseEntity.created(new URI("/event/" + eventId + "/editrubric")).body(rubric);
	}

	/***
	 * Does the necessary error checking to see if the rubric can be modified. If it
	 * cannot be modified, the appropriate response entity is returned instead.
	 * Otherwise, the changes are saved.
	 * 
	 * @param eventId id of the event to set with the rubric
	 * @param rubric  to save in the database
	 * @return URI of the new rubric or an HTTP status
	 * @throws URISyntaxException
	 */
	public ResponseEntity<?> editRubricByEvent(long eventId, Rubric rubric) throws URISyntaxException {
		// find event:
		Optional<Event> optEvent = eventRepository.findById(eventId);
		if (optEvent.isEmpty()) {
			return notFoundRE;
		}
		Event event = optEvent.get();

		// check access:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		long chairAccountId = event.getChair().getAccount().getId();
		if (user.getId() != chairAccountId) {
			return notFoundRE;
		}

		// check for previous rubric:
		if (event.getRubric() == null) {// if no previous rubric
			return this.createRubricByEvent(eventId, rubric);
		}

		// check for scored rubrics:
		Rubric oldRubric = rubricRepository.findById(event.getRubric().getId()).get();
		if (oldRubric.getScoredRubrics().size() > 0) {
			return new ResponseEntity<>(HttpStatus.UNPROCESSABLE_ENTITY);
		}

		// delete previous categories:
		while (oldRubric.getCategories().size() > 0) {
			this.deleteCategory(oldRubric.getCategories().get(0).getId());
		}
		event.setRubric(null);
		rubricRepository.deleteById(oldRubric.getId());

		// switch event to new rubric:
		rubric.setId(0);
		event.setRubric(rubric);
		//rubricRepository.save(oldRubric);

		// remove id from new categories:
		for (Category c : rubric.getCategories()) {
			c.setId(0);
			for (Category sc : c.getSubCategories()) {
				sc.setId(0);
			}
		}

		// save new rubric:
		rubric = this.saveRubric(rubric);
		return ResponseEntity.created(new URI("/event/" + eventId + "/editrubric")).body(rubric);
	}

	public Optional<Event> getEventName(long eventId) {
		Optional<Event> optEvent = eventRepository.findById(eventId);
		if ( optEvent.isEmpty() ) {
			return Optional.empty();
		}
		optEvent.get().setJudges(null);
		optEvent.get().setEntries(null);
		return optEvent;
	}
	
	/**
	 * Saves a rubric in the database. Also, saves all of the rubric's categories.
	 * Assumes the rubric is already associated with an event.
	 * 
	 * @param rubric to be saved
	 * @return saved instance of rubric for further operation.
	 */
	private Rubric saveRubric(Rubric rubric) {
		// save rubric:
		Rubric newRubric = rubricRepository.save(rubric);

		// save categories of rubric:
		List<Category> cats = newRubric.getCategories();
		for (int i = 0; i < cats.size(); i++) {
			this.saveCategory(cats.get(i));
		}

		// save again in case point values adjusted:
		newRubric = rubricRepository.save(newRubric);

		return newRubric;
	}

	/**
	 * Saves a category in the database and saves all its sub categories too.
	 * 
	 * @param category to be added
	 * @return saved instance of category for further operation.
	 */
	private Category saveCategory(Category category) {
		// save category:
		Category newCat = categoryRepository.save(category);

		// save sub categories:
		List<Category> subCats = newCat.getSubCategories();
		for (int i = 0; i < subCats.size(); i++) {
			this.saveCategory(subCats.get(i));
		}

		return newCat;
	}

	/**
	 * Deletes a category with a given id, and all its sub categories. The deletion
	 * of sub categories is done recursively.
	 * 
	 * @param id of category to delete
	 */
	private void deleteCategory(long id) {
		// get category:
		Optional<Category> c = categoryRepository.findById(id);

		// error check:
		if (c.isEmpty()) {// if nothing found
			// do nothing:
			return;
		}
		Category category = c.get();

		// remove from rubric if possible:
		if (category.getRubric() != null) {// if has rubric
			// remove relation:
			category.setRubric(null);
		}

		// remove from parent if possible:
		if (category.getParentCategory() != null) {// if has parent
			// remove relation:
			category.setParentCategory(null);
		}

		// delete sub categories:
		while (category.getSubCategories().size() > 0) {
			long scId = category.getSubCategories().get(0).getId();
			deleteCategory(scId);
		}

		// delete self:
		categoryRepository.deleteById(id);
	}

}
