package com.evaluate.services;

import com.evaluate.models.Account;
import com.evaluate.models.Chair;
import com.evaluate.models.Entry;
import com.evaluate.models.Event;
import com.evaluate.repositories.EntryRepository;
import com.evaluate.repositories.EventRepository;

import java.util.Optional;

import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

@Service
public class EntryService {

	@Autowired
	private EntryRepository entryRepository;

	@Autowired
	private EventRepository eventRepository;

	/**
	 * Gets the entry with the matching entryId, if the user is the chair of the
	 * appropriate event
	 * 
	 * @param eventId
	 * @param entryId
	 * @return Entry with matching ID
	 */
	public ResponseEntity<Entry> getEntry(long eventId, long entryId) {
		// get data:
		Optional<Event> optEvent = eventRepository.findById(eventId);
		Optional<Entry> optEntry = entryRepository.findById(entryId);
		if (optEvent.isEmpty() || optEntry.isEmpty()) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		Event event = optEvent.get();
		Entry entry = optEntry.get();

		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Chair chair = event.getChair();
		if (user.getId() != chair.getAccount().getId()// if user is NOT the chair
				|| entry.getEvent().getId() != event.getId()) {// if entry is NOT for the event
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}

		return ResponseEntity.ok(entry);
	}

	/**
	 * Modifies the entry with the matching entryId to have the attributes of the
	 * newEntry. Only allowed if user is the chair of the appropriate event.
	 * 
	 * @param eventId
	 * @param entryId
	 * @param newEntry
	 * @return newly modified entry
	 */
	public ResponseEntity<Entry> editEntry(long eventId, long entryId, Entry newEntry) {
		// get data:
		Optional<Event> optEvent = eventRepository.findById(eventId);
		Optional<Entry> optEntry = entryRepository.findById(entryId);
		if (optEvent.isEmpty() || optEntry.isEmpty()) {
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}
		Event event = optEvent.get();
		Entry oldEntry = optEntry.get();

		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Chair chair = event.getChair();
		if (user.getId() != chair.getAccount().getId()// if user is NOT the chair
				|| oldEntry.getEvent().getId() != event.getId()) {// if entry is NOT for the event
			return new ResponseEntity<>(HttpStatus.NOT_FOUND);
		}

		// modify entry:
		oldEntry.setTitle(newEntry.getTitle());
		oldEntry.setDescription(newEntry.getDescription());
		oldEntry.setTeamName(newEntry.getTeamName());
		oldEntry.setTeamLeadName(newEntry.getTeamLeadName());
		oldEntry.setTeamLeadEmail(newEntry.getTeamLeadEmail());
		oldEntry.setTeamMemberNames(newEntry.getTeamMemberNames());
		newEntry = entryRepository.save(oldEntry);

		return ResponseEntity.ok(newEntry);
	}

}
