package com.evaluate.services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import com.evaluate.models.Account;
import com.evaluate.models.Chair;
import com.evaluate.models.Event;
import com.evaluate.models.InviteCode;
import com.evaluate.models.Judge;
import com.evaluate.repositories.EventRepository;
import com.evaluate.repositories.InviteCodeRepository;

@Service
public class EventDetailService {

	private ResponseEntity<?> notFoundRE = new ResponseEntity<>(HttpStatus.NOT_FOUND);

	@Autowired
	public EventRepository eventRepository;
	@Autowired
	public InviteCodeRepository inviteCodeRepository;

	/**
	 * Returns an event with the given id. May exclude some information depending on
	 * permission level.
	 * 
	 * @param id of event to find
	 * @return ResponseEntity with corresponding data
	 */
	public ResponseEntity<?> getEvent(long id) {

		Optional<Event> optEvent = eventRepository.findById(id);
		if (optEvent.isEmpty()) {
			return notFoundRE;
		}
		Event event = optEvent.get();

		// start map:
		Map<String, Object> body = new HashMap<>();
		body.put("event", event);
		body.put("role", "anon");

		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		if (user == null) {// is anon
			event.setRubric(null);
			body.put("event", event);
			return ResponseEntity.ok().body(body);
		}

		// check if user is a judge:
		List<Judge> judges = event.getJudges();
		for (int i = 0; i < judges.size(); i++) {
			if (judges.get(i).getAccount().getId() == user.getId()) {
				body.put("role", "judge");
				body.put("judgeId", judges.get(i).getId());
				return ResponseEntity.ok().body(body);
			}
		}

		// check if user is the chair:
		Chair chair = event.getChair();
		if (user.getId() != chair.getAccount().getId()) {// if user is NOT the chair
			return notFoundRE;
		}
		body.put("role", "chair");
		body.put("judgeId", chair.getId());

		// get invite codes:
		body.put("inviteCodes", event.getInviteCodes());

		// get judges:
		List<Map<String, Object>> judgesList = new ArrayList<Map<String, Object>>();
		Map<String, Object> chairMap = new HashMap<String, Object>();
		chairMap.put("name", chair.getAccount().getFullName());
		chairMap.put("groupNumber", chair.getGroupNumber());
		judgesList.add(chairMap);
		for (int i = 0; i < judges.size(); i++) {
			Map<String, Object> judgeMap = new HashMap<String, Object>();
			judgeMap.put("name", judges.get(i).getAccount().getFullName());
			judgeMap.put("groupNumber", judges.get(i).getGroupNumber());
			judgesList.add(judgeMap);
		}
		body.put("judges", judgesList);

		// get entries:
		body.put("entries", event.getEntries());

		return ResponseEntity.ok().body(body);
	}

	/**
	 * Lets the chair generate a new unique invite code.
	 * 
	 * @param id of event
	 * @return ResponseEntity with list of invite codes containing new code
	 */
	public ResponseEntity<?> generateInviteCode(long id) {
		// get event:
		Optional<Event> optEvent = eventRepository.findById(id);
		if (optEvent.isEmpty()) {
			return notFoundRE;
		}
		Event event = optEvent.get();

		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Chair chair = event.getChair();
		if (chair.getAccount().getId() != user.getId()) {
			return notFoundRE;
		}

		// generate new code:
		InviteCode newInviteCode;
		Optional<InviteCode> optIc;
		do {
			newInviteCode = new InviteCode(event);
			optIc = inviteCodeRepository.findById(newInviteCode.getCode());
		} while (optIc.isPresent());
		inviteCodeRepository.save(newInviteCode);

		// respond with list of invite codes:
		List<InviteCode> inviteCodes = event.getInviteCodes();
		return ResponseEntity.ok(inviteCodes);
	}

	public ResponseEntity<?> deleteInviteCode(String code) {
		// get invite code:
		Optional<InviteCode> optInviteCode = inviteCodeRepository.findById(code);
		if (optInviteCode.isEmpty()) {
			return notFoundRE;
		}
		InviteCode inviteCode = optInviteCode.get();

		// check authorization:
		Account user = (Account) SecurityUtils.getSubject().getPrincipal();
		Event event = inviteCode.getEvent();
		if (event.getChair().getAccount().getId() != user.getId()) {
			return notFoundRE;
		}

		// delete invite code:
		inviteCodeRepository.deleteById(code);

		// respond with list of invite codes:
		List<InviteCode> inviteCodes = event.getInviteCodes();
		return ResponseEntity.ok(inviteCodes);
	}

}
