package com.evaluate.services;

import java.util.Optional;
import java.util.Random;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import com.evaluate.models.Account;
import com.evaluate.repositories.AccountRepository;

@Service
public class ResetService {

	private ResponseEntity<?> notFoundRE = new ResponseEntity<>(HttpStatus.NOT_FOUND);

	@Autowired
	private AccountRepository accountRepository;

	@Autowired
	private MailService mailService;

	@Autowired
	private HashService hashService;

	/**
	 * Will send a reset password email to the user if the account exists.
	 * 
	 * @return true if account exists or false otherwise
	 */
	public boolean forgotPassword(String email) {
		// check for account:
		Optional<Account> optAccount = accountRepository.findByEmail(email);
		if (optAccount.isEmpty()) {
			return false;
		}
		Account account = optAccount.get();

		// generate code:
		String code = generateCode(account);

		// send email:
		String header = "Password Reset Request for e-Valuate";
		String body = "Hello " + account.getFirstName() + ",\n\n";
		body += "Your e-Valuate password can be reset by following the link below. If you did not request a new password, please ignore this email.\n\n";
		body += MailService.domain + "resetpassword/" + code;
		mailService.sendEmail(email, header, body);
		return true;
	}

	public ResponseEntity<?> resetPassword(String code, Account newPwAccount) {
		// find account:
		Optional<Account> optAccount = accountRepository.findByEmail(newPwAccount.getEmail());
		if (optAccount.isEmpty()) {// if no account found
			return notFoundRE;
		}
		Account account = optAccount.get();

		// encrypt code:
		String encryptedCode = hashService.hash(account.getEmail(), code);

		// compare encrypted code:
		if (!encryptedCode.equals(account.getEncryptedResetCode())) {// if not match
			return notFoundRE;
		}

		// reset password and save:
		account.setEncryptedPassword(hashService.hash(account.getEmail(), newPwAccount.getEncryptedPassword()));
		account.setEncryptedResetCode("");
		accountRepository.save(account);
		return new ResponseEntity<>(HttpStatus.OK);
	}

	/**
	 * Generates a reset code, which is returned. The code is then encrypted before
	 * being set as an attribute of the account. Only the encrypted version is saved
	 * in the database. Assumes email has an existing account;
	 * 
	 * @param account to generate code for
	 * @return unencrypted reset code
	 */
	private String generateCode(Account account) {
		// get random number:
		Random r = new Random();
		String rand = r.nextInt() + "";

		// generate code:
		String code = hashService.hash(account.getEmail(), rand);
		code = code.substring(0, 15);

		// encrypt code and save:
		String toSave = hashService.hash(account.getEmail(), code);
		account.setEncryptedResetCode(toSave);
		accountRepository.save(account);

		// return unencrypted:
		return code;
	}

}
