package com.evaluate.models;

import static org.junit.jupiter.api.Assertions.*;

import java.util.*;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
class CategoryTest {
	
	private Category category;
	private List<Category> subCategories;
	private Score score;
	private Set<Score> scores;
	private Rubric rubric;
	
	@BeforeEach
	void setUp() throws Exception {
		category = new Category();
		score = new Score();
		subCategories = new ArrayList<Category>();
		for (int i = 0; i < 4; i++) {
			subCategories.add(new Category());
		}
		scores = new HashSet<Score>();
		for (int i = 0; i < 3; i++) {
			scores.add(new Score());
		}
		rubric = new Rubric();
	}
	
	@Test
	public void testCategory() {
		category = new Category();
		assertEquals("", category.getName());
		assertNull(category.getDescription());
		assertEquals(0, category.getMaxScore());
		assertNull(category.getParentCategory());
		assertEquals(new ArrayList<Category>(), category.getSubCategories());
		assertEquals(new HashSet<Score>(), category.getScores());
		assertNull(category.getRubric());
	}
	
	@Test
	public void testCategoryString() {
		//good input:
		category = new Category("Test name");
		assertEquals("Test name", category.getName());
		assertNull(category.getDescription());
		assertEquals(0, category.getMaxScore());
		assertNull(category.getParentCategory());
		assertEquals(new ArrayList<Category>(), category.getSubCategories());
		assertEquals(new HashSet<Score>(), category.getScores());
		assertNull(category.getRubric());
		
		//bad input:
		category = new Category(null);
		assertEquals("", category.getName());
		assertNull(category.getDescription());
		assertEquals(0, category.getMaxScore());
		assertNull(category.getParentCategory());
		assertEquals(new ArrayList<Category>(), category.getSubCategories());
		assertEquals(new HashSet<Score>(), category.getScores());
		assertNull(category.getRubric());
	}
	
	@Test
	public void testCategoryStringInt() {
		//good inputs:
		category = new Category("Test name", 10);
		assertEquals("Test name", category.getName());
		assertNull(category.getDescription());
		assertEquals(10, category.getMaxScore());
		assertNull(category.getParentCategory());
		assertEquals(new ArrayList<Category>(), category.getSubCategories());
		assertEquals(new HashSet<Score>(), category.getScores());
		assertNull(category.getRubric());
		
		//bad inputs:
		category = new Category(null, -1);
		assertEquals("", category.getName());
		assertNull(category.getDescription());
		assertEquals(0, category.getMaxScore());
		assertNull(category.getParentCategory());
		assertEquals(new ArrayList<Category>(), category.getSubCategories());
		assertEquals(new HashSet<Score>(), category.getScores());
		assertNull(category.getRubric());
	}
	
	@Test
	public void testCategoryStringIntCategory() {
		//good inputs:
		Category parent = new Category();
		category = new Category("Test name", 10, parent);
		assertEquals("Test name", category.getName());
		assertNull(category.getDescription());
		assertEquals(10, category.getMaxScore());
		assertEquals(parent, category.getParentCategory());
		assertTrue(parent.getSubCategories().contains(category));
		assertEquals(new ArrayList<Category>(), category.getSubCategories());
		assertEquals(new HashSet<Score>(), category.getScores());
		assertNull(category.getRubric());
		
		//bad inputs:
		category = new Category(null, -1, null);
		assertEquals("", category.getName());
		assertNull(category.getDescription());
		assertEquals(0, category.getMaxScore());
		assertNull(category.getParentCategory());
		assertEquals(new ArrayList<Category>(), category.getSubCategories());
		assertEquals(new HashSet<Score>(), category.getScores());
		assertNull(category.getRubric());
	}
	
	@Test
	public void testSetName() {
		//verify empty title at start:
 		assertEquals("", category.getName());
 		
		//Set a name
		category.setName("C1");
		assertEquals("C1", category.getName());
		
		//Change to another
		category.setName("C2");
		assertEquals("C2", category.getName());
		
		//try setting title to null:
 		category.setName(null);

 		//verify equal to empty string instead:
 		assertNotNull(category.getName());
 		assertEquals("", category.getName());
	}
	
	@Test
	public void testSetDescription() {
		//verify null description at start:
 		assertNull(category.getDescription());

 		//set description:
 		category.setDescription("Here's a description");
 		assertEquals("Here's a description", category.getDescription());

 		//change description:
 		category.setDescription("Another description");
 		assertEquals("Another description", category.getDescription());

 		//set description to null:
 		category.setDescription(null);
 		assertNull(category.getDescription());
 		
 		//set description to empty string:
 		category.setDescription("");
 		assertNull(category.getDescription());
	}
	
	@Test
	public void testSetMaxScore() {
		//Verify initial value of 0
		assertEquals(0, category.getMaxScore());
		
		//Set another value and test again
		assertTrue(category.setMaxScore(5));
		assertEquals(5, category.getMaxScore());
		
		//Reset to the same value
		assertFalse(category.setMaxScore(5));
		assertEquals(5, category.getMaxScore());
		
		//Set another value and test again
		assertTrue(category.setMaxScore(10));
		assertEquals(10, category.getMaxScore());
		
		//set a value below zero and verify no change:
		assertFalse(category.setMaxScore(-1));
		assertEquals(10, category.getMaxScore());
		
		//add 2 subcats to category and check the max score
		subCategories.get(0).setMaxScore(5);
		subCategories.get(1).setMaxScore(10);
		category.addSubCategory(subCategories.get(0));
		category.addSubCategory(subCategories.get(1));
		category.setRubric(rubric);
		assertEquals(15, category.getMaxScore());
		assertEquals(15, rubric.getTotalMaxScore());
		
		//try change parent category:
		assertFalse(category.setMaxScore(30));
		assertEquals(15, category.getMaxScore());
		
		//change the maxScore of sub category and check the maxScore for category
		subCategories.get(1).setMaxScore(20);
		assertEquals(25, category.getMaxScore());
		assertEquals(25, rubric.getTotalMaxScore());
	}
	
	@Test 
	public void testSetParentCategory() {
		//assigning ca to category
		Category ca = new Category();
		category.setParentCategory(ca);
		assertEquals(ca,category.getParentCategory());
		assertTrue(ca.getSubCategories().contains(category));
		//assigning ca twice
		category.setParentCategory(ca);
		assertEquals(ca,category.getParentCategory());
		assertEquals(1,ca.getSubCategories().size());
		assertTrue(ca.getSubCategories().contains(category));
		//assigning a new category to parent
		Category ca2 = new Category();
		category.setParentCategory(ca2);
		assertEquals(ca2,category.getParentCategory());
		//I'm sure that the old subcat has been removed, but it still has a subcatlist, which is an empty list
		//assertEquals(null,ca.getSubCategories());
	}

	@Test
	@SuppressWarnings("unchecked")
	public void testSetSubCategories() {
		//verify empty at start:
		assertNotNull(category.getSubCategories());
		assertEquals(0, category.getSubCategories().size());
		
		//set sub categories and verify relations:
		List<Category> scCopy = (List<Category>)((ArrayList<Category>)subCategories).clone();
		category.setSubCategories(scCopy);
		assertIterableEquals(subCategories, category.getSubCategories());
		for (Category sc : subCategories) {
			assertEquals(category, sc.getParentCategory());
		}
		
		//set SAME sub categories and verify relations:
		List<Category> scCopy2 = (List<Category>)((ArrayList<Category>)subCategories).clone();
		category.setSubCategories(scCopy2);
		assertIterableEquals(subCategories, category.getSubCategories());
		for (Category sc : subCategories) {
			assertEquals(category, sc.getParentCategory());
		}
		
		//create new set of sub categories
		List<Category> newSubCategories = new ArrayList<Category>();
		for (int i = 0; i < 3; i++) {
			newSubCategories.add(new Category());
		}
		
		//set new sub categories and verify relations:
		List<Category> scCopy3 = (List<Category>)((ArrayList<Category>)newSubCategories).clone();
		category.setSubCategories(scCopy3);
		assertIterableEquals(newSubCategories, category.getSubCategories());
		for (Category sc : newSubCategories) {
			assertEquals(category, sc.getParentCategory());
		}
		
		//try setting null:
		category.setSubCategories(null);
		assertNotNull(category.getSubCategories());
		assertEquals(new ArrayList<Category>(), category.getSubCategories());
	}
	
	@Test
	public void testAddSubCategory() {
		//assign ca to category
		Category ca = new Category();
		assertTrue(category.addSubCategory(ca));
		assertFalse(category.addSubCategory(ca));
		category.addSubCategory(ca);
		assertTrue(category.getSubCategories().contains(ca));
		assertEquals(category,ca.getParentCategory());
		//assign ca to category again
		category.addSubCategory(ca);
		assertTrue(category.getSubCategories().contains(ca));
		assertEquals(category,ca.getParentCategory());
		assertEquals(1,category.getSubCategories().size());
		//assign another subcat to category
		Category ca2 = new Category();
		category.addSubCategory(ca2);
		assertTrue(category.getSubCategories().contains(ca2));
		assertEquals(2,category.getSubCategories().size());
	}

	@Test
	@SuppressWarnings("unchecked")
	public void testRemoveSubCategoryInt() {
		//give parent sub categories:
		List<Category> scCopy = (List<Category>)((ArrayList<Category>)subCategories).clone();
		category.setSubCategories(scCopy);
		assertNotEquals(0, category.getSubCategories());
		
		//try to remove out of bounds:
		int scSize = category.getSubCategories().size();
		assertNull(category.removeSubCategory(-1));
		assertNull(category.removeSubCategory(scSize));
		assertEquals(scSize, category.getSubCategories().size());
		
		//remove one by one:
		for (int i = 0; i < subCategories.size(); i++) {
			Category wasRemoved = category.removeSubCategory(0);
			assertNull(wasRemoved.getParentCategory());
		}
		assertIterableEquals(new ArrayList<Category>(), category.getSubCategories());
		
		//give parent sub categories:
		List<Category> scCopy2 = (List<Category>)((ArrayList<Category>)subCategories).clone();
		category.setSubCategories(scCopy2);
		assertNotEquals(0, category.getSubCategories());
		
		//remove one by one differently:
		scSize = category.getSubCategories().size();
		for (int i = scSize-1; i >= 0; i--) {
			Category wasRemoved = category.removeSubCategory(i);
			assertNull(wasRemoved.getParentCategory());
		}
		assertIterableEquals(new ArrayList<Category>(), category.getSubCategories());
	}

	@Test
	@SuppressWarnings("unchecked")
	public void testRemoveSubCategoryCategory() {
		//give parent sub categories:
		List<Category> scCopy = (List<Category>)((ArrayList<Category>)subCategories).clone();
		category.setSubCategories(scCopy);
		assertNotEquals(0, category.getSubCategories());
		
		//try removing null:
		int scSize = subCategories.size();
		assertFalse(category.removeSubCategory(null));
		assertEquals(scSize, category.getSubCategories().size());
		
		//try removing category not in list:
		assertFalse(category.removeSubCategory(new Category()));
		assertEquals(scSize, category.getSubCategories().size());
		
		//remove one by one:
		for (Category sc : subCategories) {
			assertTrue(category.removeSubCategory(sc));
			assertNull(sc.getParentCategory());
		}
		assertIterableEquals(new ArrayList<Category>(), category.getSubCategories());

		//give parent sub categories:
		List<Category> scCopy2 = (List<Category>)((ArrayList<Category>)subCategories).clone();
		category.setSubCategories(scCopy2);
		assertNotEquals(0, category.getSubCategories());
		
		//remove one by one in different order:
		int scInitialSize = category.getSubCategories().size();
		for (int i = scInitialSize-1; i >= 0; i--) {
			Category toRemove = subCategories.get(i);
			assertTrue(category.removeSubCategory(toRemove));
			assertNull(toRemove.getParentCategory());
		}
		assertEquals(new ArrayList<>(), category.getSubCategories());
	}
	
	@SuppressWarnings("unchecked")
	@Test
	public void testSetScores() {
		//verify empty at start:
		assertEquals(new HashSet<Score>(), category.getScores());
		
		//set scores and verify relations:
		Set<Score> scCopy = (Set<Score>)((HashSet<Score>)scores).clone();
		category.setScores(scCopy);
		assertEquals(scores, category.getScores());
		for (Score sc : scores) {
			assertEquals(category, sc.getTemplate());
		}
		
		//set SAME scores and verify relations:
		Set<Score> scCopy2 = (Set<Score>)((HashSet<Score>)scores).clone();
		category.setScores(scCopy2);
		assertEquals(scores, category.getScores());
		for (Score sc : scores) {
			assertEquals(category, sc.getTemplate());
		}
		
		//create new set of scores
		Set<Score> newScores = new HashSet<Score>();
		for (int i = 0; i < 3; i++) {
			newScores.add(new Score());
		}
		
		//set new scores and verify relations:
		Set<Score> scCopy3 = (Set<Score>)((HashSet<Score>)scores).clone();
		category.setScores(scCopy3);
		assertEquals(scores, category.getScores());
		for (Score sc : scores) {
			assertEquals(category, sc.getTemplate());
		}
		
		//try setting null:
		category.setScores(null);
		assertNotNull(category.getScores());
		assertEquals(new HashSet<Score>(), category.getScores());
	}
	
	@Test
	public void testAddScore() {
		//verify empty
		assertEquals(new HashSet<Score>(), category.getScores());
		
		//add a score and check
		assertTrue(category.addScore(score));
		assertEquals(1, category.getScores().size());
		assertEquals(category, score.getTemplate());
		assertTrue(category.getScores().contains(score));
		
		//add THE SAME score and check
		assertFalse(category.addScore(score));
		assertEquals(1, category.getScores().size());
		
		//add another score
		Score sc2 = new Score();
		assertTrue(category.addScore(sc2));
		assertEquals(2, category.getScores().size());
		assertEquals(category, sc2.getTemplate());
		assertTrue(category.getScores().contains(sc2));
		
		//add a null
		assertFalse(category.addScore(null));
		assertEquals(2, category.getScores().size());
	}
	
	@Test
	@SuppressWarnings("unchecked")
	public void testRemoveScore() {
		//assign scores to category:
		Set<Score> sCopy = (Set<Score>)((HashSet<Score>)scores).clone();
		category.setScores(sCopy);
		assertNotEquals(0, category.getScores().size());
		
		//try removing null:
		assertFalse(category.removeScore(null));
		assertEquals(scores.size(), category.getScores().size());
		
		//try removing non existent score:
		assertFalse(category.removeScore(new Score()));
		assertEquals(scores.size(), category.getScores().size());
		
		//remove one by one:
		for (Score s : scores) {
			assertTrue(category.removeScore(s));
			assertNull(s.getTemplate());
		}
		assertEquals(new HashSet<Score>(), category.getScores());
	}
	
	@Test
	public void testSetRubric() {
		//verify no rubric at start:
		assertNull(category.getRubric());
		
		//set rubric:
		category.setRubric(rubric);
		assertEquals(rubric, category.getRubric());
		assertTrue(rubric.getCategories().contains(category));
		
		//set SAME rubric:
		int size = rubric.getCategories().size();
		category.setRubric(rubric);
		assertEquals(rubric, category.getRubric());
		assertTrue(rubric.getCategories().contains(category));
		assertEquals(size, rubric.getCategories().size());
		
		//set to different rubric:
		Rubric newRubric = new Rubric();
		category.setRubric(newRubric);
		assertEquals(newRubric, category.getRubric());
		assertTrue(newRubric.getCategories().contains(category));
		
		//verify old relations removed:
		assertNotEquals(rubric, category.getRubric());
		assertFalse(rubric.getCategories().contains(category));
		
		//set to null:
		category.setRubric(null);
		assertNull(category.getRubric());
		assertFalse(newRubric.getCategories().contains(category));
	}
	
}
