package com.evaluate.models;

import static org.junit.jupiter.api.Assertions.*;

import java.util.HashSet;
import java.util.Set;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
class RoleTest {

	private Role role;
	private Account account;
	private Set<ScoredRubric> scoredRubrics;
	private ScoredRubric scoredRubric;

	@BeforeEach
	void setUp() throws Exception {
		role = new Judge();

		account = new Account();
		account.setFirstName("Mike");
		account.setLastName("Scherger");

		scoredRubrics = new HashSet<ScoredRubric>();
		for (int i = 0; i < 4; i++) {
			scoredRubrics.add(new ScoredRubric());
		}
		scoredRubric = new ScoredRubric();
	}

	@Test
	void testRole() {
		role = new Judge();
		assertNull(role.getAccount());
		assertNull(role.getEvent());
		assertEquals(0, role.getGroupNumber());
		assertEquals(new HashSet<ScoredRubric>(), role.getScoredRubrics());

		role = new Chair();
		assertNull(role.getAccount());
		assertNull(role.getEvent());
		assertEquals(-1, role.getGroupNumber());
		assertEquals(new HashSet<ScoredRubric>(), role.getScoredRubrics());
	}

	@Test
	void testSetAccount() {
		// verify null at first
		assertNull(role.getAccount());

		// set account
		role.setAccount(account);
		assertEquals(account, role.getAccount());
		assertTrue(account.getRoles().contains(role));

		// set SAME account
		role.setAccount(account);
		assertEquals(account, role.getAccount());
		assertTrue(account.getRoles().contains(role));
		assertEquals(1, account.getRoles().size());

		// set different account
		Account account2 = new Account();
		role.setAccount(account2);
		assertEquals(account2, role.getAccount());
		assertTrue(account2.getRoles().contains(role));
		assertFalse(account.getRoles().contains(role));

		// set null
		role.setAccount(null);
		assertNull(role.getAccount());
		assertFalse(account2.getRoles().contains(role));
	}

	@Test
	@SuppressWarnings("unchecked")
	void testSetScoredRubrics() {
		// verify empty at start
		assertEquals(new HashSet<ScoredRubric>(), role.getScoredRubrics());

		// set scored rubrics
		Set<ScoredRubric> srCopy = (Set<ScoredRubric>) ((HashSet<ScoredRubric>) scoredRubrics).clone();
		role.setScoredRubric(srCopy);
		assertEquals(scoredRubrics, role.getScoredRubrics());
		for (ScoredRubric sr : role.getScoredRubrics()) {
			assertEquals(role, sr.getJudge());
		}

		// set SAME scored rubrics
		Set<ScoredRubric> srCopy2 = (Set<ScoredRubric>) ((HashSet<ScoredRubric>) scoredRubrics).clone();
		role.setScoredRubric(srCopy2);
		assertEquals(scoredRubrics, role.getScoredRubrics());
		for (ScoredRubric sr : role.getScoredRubrics()) {
			assertEquals(role, sr.getJudge());
		}

		// set different scored rubrics
		Set<ScoredRubric> diffScoredRubrics = new HashSet<ScoredRubric>();
		for (int i = 0; i < 3; i++) {
			diffScoredRubrics.add(new ScoredRubric());
		}
		Set<ScoredRubric> dsrCopy = (Set<ScoredRubric>) ((HashSet<ScoredRubric>) diffScoredRubrics).clone();
		role.setScoredRubric(dsrCopy);
		assertEquals(diffScoredRubrics, role.getScoredRubrics());
		for (ScoredRubric dsr : diffScoredRubrics) {
			assertEquals(role, dsr.getJudge());
		}

		// verify old relations removed
		for (ScoredRubric sr : scoredRubrics) {
			assertNull(sr.getJudge());
		}

		// set null
		role.setScoredRubric(null);
		assertEquals(new HashSet<ScoredRubric>(), role.getScoredRubrics());
		for (ScoredRubric dsr : diffScoredRubrics) {
			assertNull(dsr.getJudge());
		}
	}

	@Test
	void testAddScoredRubric() {
		// verify empty at start:
		assertEquals(new HashSet<ScoredRubric>(), role.getScoredRubrics());

		// add scored rubric
		assertTrue(role.addScoredRubric(scoredRubric));
		assertTrue(role.getScoredRubrics().contains(scoredRubric));
		assertEquals(role, scoredRubric.getJudge());

		// add same scored rubric
		int size = role.getScoredRubrics().size();
		assertFalse(role.addScoredRubric(scoredRubric));
		assertEquals(size, role.getScoredRubrics().size());
		assertEquals(role, scoredRubric.getJudge());

		// add new scored rubric
		ScoredRubric diffScoredRubric = new ScoredRubric();
		assertTrue(role.addScoredRubric(diffScoredRubric));
		assertTrue(role.getScoredRubrics().contains(diffScoredRubric));
		assertEquals(size + 1, role.getScoredRubrics().size());
		assertEquals(role, diffScoredRubric.getJudge());

		// set up with previous scored rubrics
		role.setScoredRubric(scoredRubrics);
		size = role.getScoredRubrics().size();

		// add scored rubric
		assertTrue(role.addScoredRubric(scoredRubric));
		assertTrue(role.getScoredRubrics().contains(scoredRubric));
		assertEquals(size + 1, role.getScoredRubrics().size());
		assertEquals(role, scoredRubric.getJudge());

		// try adding null
		assertFalse(role.addScoredRubric(null));
		assertEquals(size + 1, role.getScoredRubrics().size());
	}

	void testRemoveScoredRubric() {
		// give rubric scored rubrics
		role.setScoredRubric(scoredRubrics);
		assertNotEquals(0, role.getScoredRubrics().size());

		// try removing scored rubric not in set
		assertFalse(role.removeScoredRubric(new ScoredRubric()));

		// try removing null
		assertFalse(role.removeScoredRubric(null));

		// remove one by one
		for (ScoredRubric sr : scoredRubrics) {
			assertTrue(role.removeScoredRubric(sr));
			assertNull(sr.getJudge());
		}
		assertEquals(new HashSet<ScoredRubric>(), role.getScoredRubrics());
	}

}
