package com.evaluate.models;

import static org.junit.jupiter.api.Assertions.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
class RubricTest {
	
	private Rubric rubric;
	private Event event;
	private ScoredRubric scoredRubric;
	private Set<ScoredRubric> scoredRubrics;
	private Category category;
	private List<Category> categories;

	@BeforeEach
	void setUp() throws Exception {
		rubric = new Rubric();
		event = new Event();
		scoredRubric = new ScoredRubric();
		scoredRubrics = new HashSet<ScoredRubric>();
		for (int i = 0; i < 4; i++) {
			scoredRubrics.add(new ScoredRubric());
		}
		category = new Category();
		categories = new ArrayList<Category>();
		for (int i = 0; i < 4; i++) {
			categories.add(new Category());
		}
	}
	
	@Test
	void testRubric() {
		rubric = new Rubric();
		assertNull(rubric.getEvent());
		assertEquals(new ArrayList<Category>(), rubric.getCategories());
		assertEquals(new HashSet<ScoredRubric>(), rubric.getScoredRubrics());
		assertEquals(0, rubric.getTotalMaxScore());
	}
	
	@Test
	@SuppressWarnings("unchecked")
	void testRubricListOfCategorySetOfScoredRubric() {
		assertNull(rubric.getEvent());
		List<Category> cCopy = (List<Category>)((ArrayList<Category>)categories).clone();
		Set<ScoredRubric> srCopy = (Set<ScoredRubric>)((HashSet<ScoredRubric>)scoredRubrics).clone();
		rubric = new Rubric(cCopy, srCopy);
		assertIterableEquals(cCopy, rubric.getCategories());
		for (Category c : categories) {
			assertEquals(rubric, c.getRubric());
		}
		assertEquals(srCopy, rubric.getScoredRubrics());
		for (ScoredRubric sr : scoredRubrics) {
			assertEquals(rubric, sr.getTemplate());
		}
		assertEquals(0, rubric.getTotalMaxScore());
	}
	
	@Test
	void testSetEvent() {
		//verify null at first
		assertNull(rubric.getEvent());
		
		//set event
		rubric.setEvent(event);
		assertEquals(event, rubric.getEvent());
		assertEquals(rubric, event.getRubric());
		
		//set SAME event
		rubric.setEvent(event);
		assertEquals(event, rubric.getEvent());
		assertEquals(rubric, event.getRubric());
		
		//set different event
		Event event2 = new Event();
		rubric.setEvent(event2);
		assertEquals(event2, rubric.getEvent());
		assertEquals(rubric, event2.getRubric());
		assertNull(event.getRubric());
		
		//set null
		rubric.setEvent(null);
		assertNull(rubric.getEvent());
		assertNull(event2.getRubric());
	}
	
	@Test
	@SuppressWarnings("unchecked")
	void testSetCategories() {
		//verify empty category list at first:
		assertEquals(new ArrayList<Category>(), rubric.getCategories());
		
		//set categories:
		List<Category> caCopy = (List<Category>)((ArrayList<Category>)categories).clone();
		rubric.setCategories(caCopy);
		
		//verify relations:
		assertEquals(categories, rubric.getCategories());
		for (Category ca : categories) {
			assertEquals(rubric, ca.getRubric());
		}
		
		//set SAME category:
		List<Category> caCopy2 = (List<Category>)((ArrayList<Category>)categories).clone();
		rubric.setCategories(caCopy2);
		
		//verify relations still exists:
		assertEquals(categories, rubric.getCategories());
		for (Category ca : categories) {
			assertEquals(rubric, ca.getRubric());
		}
		
		//set different Category:
		ArrayList<Category> newCategories = new ArrayList<Category>();
		for (int i = 0; i < 3; i++) {
			newCategories.add(new Category());
		}
		ArrayList<Category> caCopy3 = (ArrayList<Category>)newCategories.clone();
		rubric.setCategories(caCopy3);
		
		//verify relations set:
		assertEquals(newCategories, rubric.getCategories());
		for (Category ca : newCategories) {
			assertEquals(rubric, ca.getRubric());
		}
		
		//verify old relations removed:
		for (Category ca : categories) {
			assertNull(ca.getRubric());
			assertFalse(rubric.getCategories().contains(ca));
		}
		
		//set null:
		rubric.setCategories(null);
		
		//verify not null:
		assertEquals(new ArrayList<Category>(), rubric.getCategories());
		
		//verify relations gone:
		for (Category ca : newCategories) {
			assertNull(ca.getRubric());
		}
	}

	@Test
	void testAddCategory() {
		//verify empty at start:
		assertEquals(new ArrayList<Category>(), rubric.getCategories());
		
		//add category:
		assertTrue(rubric.addCategory(category));
		
		//verify relations
		assertTrue(rubric.getCategories().contains(category));
		assertEquals(rubric, category.getRubric());
		
		//try adding SAME category:
		assertFalse(rubric.addCategory(category));
		
		//verify size did not change:
		assertEquals(1, rubric.getCategories().size());
		assertTrue(rubric.getCategories().contains(category));
		assertEquals(rubric, category.getRubric());
		
		//add another category and verify:
		Category newCa = new Category();
		assertTrue(rubric.addCategory(newCa));
		assertEquals(2, rubric.getCategories().size());
		assertTrue(rubric.getCategories().contains(newCa));
		assertEquals(rubric, newCa.getRubric());
		
		//set up rubric with previous set of category:
		rubric.setCategories(categories);
		int caSize = rubric.getCategories().size();
		
		//add another category:
		assertTrue(rubric.addCategory(category));
		assertEquals(caSize+1, rubric.getCategories().size());
		assertTrue(rubric.getCategories().contains(category));
		assertEquals(rubric, category.getRubric());
		
		//try adding a null:
		assertFalse(rubric.addCategory(null));
		
		//verify size didn't change:
		assertEquals(caSize+1, rubric.getCategories().size());
	}
	
	@Test
	void testRemoveCategoryInt() {
		//give rubric categories:
		rubric.setCategories(categories);
		assertNotEquals(0, rubric.getCategories().size());
		
		//remove one by one:
		while (rubric.getCategories().size() > 0) {
			rubric.removeCategory(0);
		}
		
		//verify empty:
		assertEquals(new ArrayList<Category>(), rubric.getCategories());
		
		//give rubric categories:
		rubric.setCategories(categories);
		assertNotEquals(0, rubric.getCategories().size());
		
		//try removing out of bounds:
		int caSize = rubric.getCategories().size();
		assertNull(rubric.removeCategory(-1));
		assertEquals(caSize, rubric.getCategories().size());
		assertNull(rubric.removeCategory(caSize+1));
		assertEquals(caSize, rubric.getCategories().size());
	}
	
	@Test
	void testRemoveCategoryCategory() {
		//give rubric categories:
		rubric.setCategories(categories);
		assertNotEquals(0, rubric.getCategories().size());
		
		//try removing category that doesn't exist:
		int caSize = rubric.getCategories().size();
		assertFalse(rubric.removeCategory(new Category()));
		assertEquals(caSize, rubric.getCategories().size());
		
		//try removing null:
		assertFalse(rubric.removeCategory(null));
		assertEquals(caSize, rubric.getCategories().size());

		//remove one by one:
		for (Category ca : categories) {
			assertTrue(rubric.removeCategory(ca));
			assertNull(ca.getRubric());
		}
		assertEquals(new ArrayList<Category>(), rubric.getCategories());
	}

	@Test
	@SuppressWarnings("unchecked")
	void testSetScoredRubrics() {
		//verify empty scored rubric set at first:
		assertEquals(new HashSet<ScoredRubric>(), rubric.getScoredRubrics());
		
		//set scored rubrics:
		Set<ScoredRubric> srCopy = (Set<ScoredRubric>)((HashSet<ScoredRubric>)scoredRubrics).clone();
		rubric.setScoredRubrics(srCopy);
		
		//verify relations:
		assertEquals(scoredRubrics, rubric.getScoredRubrics());
		for (ScoredRubric sr : scoredRubrics) {
			assertEquals(rubric, sr.getTemplate());
		}
		
		//set SAME scored rubrics:
		Set<ScoredRubric> srCopy2 = (Set<ScoredRubric>)((HashSet<ScoredRubric>)scoredRubrics).clone();
		rubric.setScoredRubrics(srCopy2);
		
		//verify relations still exists:
		assertEquals(scoredRubrics, rubric.getScoredRubrics());
		for (ScoredRubric sr : scoredRubrics) {
			assertEquals(rubric, sr.getTemplate());
		}
		
		//set different scored rubrics:
		HashSet<ScoredRubric> newScoredRubrics = new HashSet<ScoredRubric>();
		for (int i = 0; i < 3; i++) {
			newScoredRubrics.add(new ScoredRubric());
		}
		HashSet<ScoredRubric> srCopy3 = (HashSet<ScoredRubric>)newScoredRubrics.clone();
		rubric.setScoredRubrics(srCopy3);
		
		//verify relations set:
		assertEquals(newScoredRubrics, rubric.getScoredRubrics());
		for (ScoredRubric sr : newScoredRubrics) {
			assertEquals(rubric, sr.getTemplate());
		}
		
		//verify old relations removed:
		for (ScoredRubric sr : scoredRubrics) {
			assertNull(sr.getTemplate());
			assertFalse(rubric.getScoredRubrics().contains(sr));
		}
		
		//set null:
		rubric.setScoredRubrics(null);
		
		//verify not null:
		assertEquals(new HashSet<ScoredRubric>(), rubric.getScoredRubrics());
		
		//verify relations gone:
		for (ScoredRubric sr : newScoredRubrics) {
			assertNull(sr.getTemplate());
		}
	}
	
	@Test
	void testAddScoredRubric() {
		//verify empty at start:
		assertEquals(new HashSet<ScoredRubric>(), rubric.getScoredRubrics());
		
		//add scored rubric and verify:
		assertTrue(rubric.addScoredRubric(scoredRubric));
		assertTrue(rubric.getScoredRubrics().contains(scoredRubric));
		assertEquals(rubric, scoredRubric.getTemplate());
		
		//try adding SAME scored rubric:
		assertFalse(rubric.addScoredRubric(scoredRubric));
		
		//verify size did not change:
		assertEquals(1, rubric.getScoredRubrics().size());
		assertTrue(rubric.getScoredRubrics().contains(scoredRubric));
		assertEquals(rubric, scoredRubric.getTemplate());
		
		//add new scored rubric and verify:
		ScoredRubric newSR = new ScoredRubric();
		assertTrue(rubric.addScoredRubric(newSR));
		assertEquals(2, rubric.getScoredRubrics().size());
		assertTrue(rubric.getScoredRubrics().contains(scoredRubric));
		assertEquals(rubric, scoredRubric.getTemplate());
		
		//set up rubric with previous set of scored rubric:
		rubric.setScoredRubrics(scoredRubrics);
		int srSize = rubric.getScoredRubrics().size();
		
		//add another scored rubric:
		assertTrue(rubric.addScoredRubric(scoredRubric));
		assertEquals(srSize+1, rubric.getScoredRubrics().size());
		assertTrue(rubric.getScoredRubrics().contains(scoredRubric));
		assertEquals(rubric, scoredRubric.getTemplate());
		
		//try adding a null:
		srSize = rubric.getScoredRubrics().size();
		assertFalse(rubric.addScoredRubric(null));
		
		//verify size didn't change:
		assertEquals(srSize, rubric.getScoredRubrics().size());
	}

	@Test
	void testRemoveScoredRubric() {
		//give rubric scored rubrics:
		rubric.setScoredRubrics(scoredRubrics);
		assertNotEquals(0, rubric.getScoredRubrics().size());
		
		//try removing non existent scored rubric:
		assertFalse(rubric.removeScoredRubric(new ScoredRubric()));
		
		//try removing null:
		assertFalse(rubric.removeScoredRubric(null));

		//remove one by one:
		for (ScoredRubric sr : scoredRubrics) {
			assertTrue(rubric.removeScoredRubric(sr));
			assertNull(sr.getTemplate());
		}
		assertEquals(new HashSet<ScoredRubric>(), rubric.getScoredRubrics());
	}

}
